/*
 *******************************************************************************
 *
 *  Copyright 2023 RIEGL Laser Measurement Systems
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  SPDX-License-Identifier: Apache-2.0
 *
 *******************************************************************************
 */
/*!
 *******************************************************************************
 *
 * \author  RIEGL LMS GmbH, Austria
 * \brief   Description of RIEGL RDB 2 database meta data items.
 * \version 2015-10-27/AW: Initial version
 * \version 2015-11-25/AW: Item "Geo Tag" added
 * \version 2016-10-27/AW: Item "Voxel Information" added
 * \version 2016-11-17/AW: Item "Voxel Information" updated
 * \version 2016-12-12/AW: Item "Range Statistics" added
 * \version 2017-03-08/AW: Item "Plane Patch Statistics" added
 * \version 2017-04-05/AW: Items "Atmosphere" and "Geometric Scale Factor" added
 * \version 2017-08-22/AW: Items for waveform sample block and value files added
 * \version 2017-10-24/AW: Item "Gaussian Decomposition" added
 * \version 2017-11-13/AW: Item "riegl.scan_pattern" updated
 * \version 2017-11-21/AW: Item "riegl.trajectory_info" added
 * \version 2018-01-11/AW: Item "riegl.beam_geometry" added
 * \version 2018-01-15/AW: Item "riegl.reflectance_calculation" added
 * \version 2018-01-15/AW: Item "riegl.near_range_correction" added
 * \version 2018-01-15/AW: Item "riegl.device_geometry" added
 * \version 2018-02-13/AW: Item "riegl.notch_filter" added
 * \version 2018-03-08/AW: Item "riegl.window_echo_correction" added
 * \version 2018-03-15/AW: Item "riegl.pulse_position_modulation" added
 * \version 2018-05-24/AW: Item "riegl.pixel_info" added
 * \version 2018-06-08/AW: Item "riegl.shot_info" added
 * \version 2018-06-08/AW: Item "riegl.echo_info" added
 * \version 2018-06-14/AW: Item "riegl.mta_settings" added
 * \version 2018-06-14/AW: Item "riegl.receiver_internals" added
 * \version 2018-06-14/AW: Item "riegl.device_output_limits" added
 * \version 2018-06-26/AW: Schema: replace "number" with "integer" if applicable
 * \version 2018-07-09/AW: Item "riegl.pose_estimation" added
 * \version 2018-07-09/AW: Item "riegl.pose_sensors" added
 * \version 2018-09-20/AW: Item "riegl.pointcloud_info" added
 * \version 2018-11-08/AW: Item "riegl.scan_pattern" updated
 * \version 2018-11-13/AW: Item "riegl.receiver_internals" updated
 * \version 2019-03-06/AW: Item "riegl.receiver_internals" updated
 * \version 2019-03-21/AW: Item "riegl.device_geometry" updated
 * \version 2019-04-15/AW: Item "riegl.point_attribute_groups" added
 * \version 2019-04-30/AW: Item "riegl.waveform_settings" added
 * \version 2019-10-03/AW: Item "riegl.angular_notch_filter" added
 * \version 2019-10-03/AW: Item "riegl.noise_estimates" added
 * \version 2019-10-25/AW: Item "riegl.window_analysis" added
 * \version 2019-11-06/AW: Item "riegl.georeferencing_parameters" added
 * \version 2019-11-27/AW: Item "riegl.plane_patch_matching" added
 * \version 2019-12-13/AW: Items for tie-/control objects added
 * \version 2019-12-19/AW: Items for tie-/control objects added
 * \version 2020-02-04/AW: Item "riegl.detection_probability" added
 * \version 2020-02-04/AW: Item "riegl.licenses" added
 * \version 2020-04-27/AW: Item "riegl.waveform_info" updated (schema+example)
 * \version 2020-09-03/AW: Item "riegl.reflectance_correction" added
 * \version 2020-09-10/AW: Item "riegl.device_geometry_passive_channel" added
 * \version 2020-09-25/AW: Item "riegl.georeferencing_parameters" updated
 * \version 2020-09-25/AW: Item "riegl.trajectory_info" updated
 * \version 2020-09-29/AW: Item "riegl.temperature_calculation" added
 * \version 2020-10-23/AW: Item "riegl.exponential_decomposition" added (#3709)
 * \version 2020-11-30/AW: Item "riegl.notch_filter" updated (schema)
 * \version 2020-12-02/AW: Item "riegl.georeferencing_parameters" updated (schema)
 * \version 2021-02-02/AW: Item "riegl.plane_slope_class_info" added (rdbplanes/#7)
 * \version 2021-02-16/AW: Item "riegl.device_output_limits" updated (schema, #3811)
 * \version 2021-03-03/AW: Item "riegl.exponential_decomposition" updated (schema, #3822)
 * \version 2021-03-03/AW: Item "riegl.waveform_averaging_settings" added (#3821)
 * \version 2021-04-01/AW: Item "riegl.voxel_info" updated (schema, #3854)
 * \version 2021-04-16/AW: Item "riegl.voxel_info" updated (schema, #3866)
 * \version 2021-09-30/AW: Item "riegl.waveform_info" updated (schema+example, #4016)
 * \version 2021-10-04/AW: Improved spelling of the descriptions of some items
 * \version 2021-11-04/AW: Rename "riegl.record_names" to "riegl.item_names" (#4034)
 * \version 2022-03-11/AW: Item "riegl.devices" added (#4162)
 * \version 2022-03-14/AW: Item "riegl.stored_filters" added (#4164)
 * \version 2022-09-20/AW: Fix typos in schema of "riegl.georeferencing_parameters"
 * \version 2022-09-30/AW: Item "riegl.system_description" added (#4368)
 * \version 2022-10-10/AW: Correction of typos in the titles of some entries
 * \version 2022-10-20/AW: Item "riegl.stored_filters" updated (example, #4164)
 * \version 2023-01-19/AW: Item "riegl.ttip_configuration" added (#4449)
 *
 *******************************************************************************
 */

#ifndef RDB_81EBCB65_FE94_471E_8D25_2B4D71CC91B1
#define RDB_81EBCB65_FE94_471E_8D25_2B4D71CC91B1

// Meta data item definition version
const char* const RDB_META_DATA_ITEMS_VERSION = "1.1.54";
const char* const RDB_META_DATA_ITEMS_DATE = "2023-01-19";

// Meta data item states
const unsigned char RDB_STATUS_REQUIRED = 1; // mandatory item
const unsigned char RDB_STATUS_OPTIONAL = 2; // optional item

// Container record for meta data item details
typedef struct MetaDataItemInfo
{
    const char*   name;
    const char*   title;
    const char*   description;
    unsigned char status;
    const char*   schema;
    const char*   example;
} MetaDataItemInfo;

// Angular notch filter parameters for window glass echoes
const MetaDataItemInfo RDB_RIEGL_ANGULAR_NOTCH_FILTER = {
    /* name        */ "riegl.angular_notch_filter",
    /* title       */ "Angular Notch Filter",
    /* description */ "Angular notch filter parameters for window glass echoes",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"required\":[\"angle\",\"range_mean\",\"amplitude_mean\"],\"type\":\"object\",\"$"
    "schema\":\"http://json-schema.org/draft-04/schema#\",\"title\":\"Angular "
    "Notch Filter\",\"description\":\"Angular notch filter parameters for "
    "window glass "
    "echoes\",\"properties\":{\"angle\":{\"type\":\"array\",\"description\":\"Angle [de"
    "g]\",\"items\":{\"type\":\"number\"}},\"amplitude_mean\":{\"type\":\"array\",\"descr"
    "iption\":\"Mean amplitude [dB]\",\"items\":{\"type\":\"number\"}},\"range_mean\":"
    "{\"type\":\"array\",\"description\":\"Mean range "
    "[m]\",\"items\":{\"type\":\"number\"}}}}"
    ,
    /* example: */
    "{\"angle\":[14.0,15.0,16.0,17.0,18.0,19.0,20.0,21.0,22.0,23.0,24.0],\"amp"
    "litude_mean\":[3.780913,3.780913,3.480913,3.120913,2.850913,2.720913,2."
    "680913,2.610913,2.530913,2.570913,2.570913],\"range_mean\":[0.094,0.094,"
    "0.09075,0.08675,0.0815,0.0775,0.074,0.071,0.068,0.0675,0.06475]}"
};

// Atmospheric parameters
const MetaDataItemInfo RDB_RIEGL_ATMOSPHERE = {
    /* name        */ "riegl.atmosphere",
    /* title       */ "Atmosphere",
    /* description */ "Atmospheric parameters",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"required\":[\"temperature\",\"pressure\",\"rel_humidity\",\"pressure_sl\",\"am"
    "sl\",\"group_velocity\",\"attenuation\",\"wavelength\"],\"type\":\"object\",\"$sch"
    "ema\":\"http://json-schema.org/draft-04/schema#\",\"title\":\"Atmospheric "
    "Parameters\",\"description\":\"Atmospheric parameters\",\"properties\":{\"temp"
    "erature\":{\"type\":\"number\",\"description\":\"Temperature along measurement"
    " path "
    "[\\u00b0C]\"},\"group_velocity\":{\"type\":\"number\",\"description\":\"Group "
    "velocity of laser beam "
    "[m/s]\"},\"rel_humidity\":{\"type\":\"number\",\"description\":\"Relative "
    "humidity along measurement path "
    "[%]\"},\"amsl\":{\"type\":\"number\",\"description\":\"Height above mean sea "
    "level (AMSL) [m]\"},\"wavelength\":{\"type\":\"number\",\"description\":\"Laser "
    "wavelength "
    "[nm]\"},\"pressure_sl\":{\"type\":\"number\",\"description\":\"Atmospheric "
    "pressure at sea level "
    "[mbar]\"},\"attenuation\":{\"type\":\"number\",\"description\":\"Atmospheric "
    "attenuation "
    "[1/km]\"},\"pressure\":{\"type\":\"number\",\"description\":\"Pressure along "
    "measurment path [mbar]\"}}}"
    ,
    /* example: */
    "{\"temperature\":7,\"group_velocity\":299711000.0,\"rel_humidity\":63,\"amsl\""
    ":0,\"wavelength\":1550,\"pressure_sl\":970,\"attenuation\":0.028125,\"pressur"
    "e\":970}"
};

// Laser beam geometry details
const MetaDataItemInfo RDB_RIEGL_BEAM_GEOMETRY = {
    /* name        */ "riegl.beam_geometry",
    /* title       */ "Beam Geometry",
    /* description */ "Laser beam geometry details",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"required\":[\"beam_exit_diameter\",\"beam_divergence\"],\"type\":\"object\",\""
    "$schema\":\"http://json-schema.org/draft-04/schema#\",\"title\":\"Beam "
    "Geometry\",\"description\":\"Laser beam geometry details\",\"properties\":{\"b"
    "eam_exit_diameter\":{\"type\":\"number\",\"description\":\"Beam width at exit "
    "aperture [m]\",\"exclusiveMinimum\":false,\"minimum\":0},\"beam_divergence\":"
    "{\"type\":\"number\",\"description\":\"Beam divergence in far field "
    "[rad]\",\"exclusiveMinimum\":false,\"minimum\":0}}}"
    ,
    /* example: */
    "{\"beam_exit_diameter\":0.0072,\"beam_divergence\":0.0003}"
};

// List of control object type definitions
const MetaDataItemInfo RDB_RIEGL_CONTROL_OBJECT_CATALOG = {
    /* name        */ "riegl.control_object_catalog",
    /* title       */ "Control Object Catalog",
    /* description */ "List of control object type definitions",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"required\":[\"types\"],\"type\":\"object\",\"$schema\":\"http://json-schema.or"
    "g/draft-04/schema#\",\"title\":\"Control Object "
    "Catalog\",\"description\":\"List of control object type definitions\",\"defi"
    "nitions\":{\"chevron\":{\"type\":\"object\",\"description\":\"chevron\",\"allOf\":["
    "{\"$ref\":\"#/definitions/common\"},{\"required\":[\"shape\",\"outside_edge_len"
    "gth\",\"thickness\"],\"type\":\"object\",\"description\":\"chevron specific prop"
    "erties\",\"properties\":{\"outside_edge_length\":{\"type\":\"number\",\"descript"
    "ion\":\"length of the two outer edges in meters\",\"exclusiveMinimum\":true"
    ",\"minimum\":0},\"shape\":{\"type\":\"string\",\"enum\":[\"chevron\"],\"description"
    "\":\"shape "
    "identifier\"},\"thickness\":{\"type\":\"number\",\"description\":\"thickness in "
    "meters\",\"exclusiveMinimum\":true,\"minimum\":0}}}]},\"cylinder\":{\"type\":\"o"
    "bject\",\"description\":\"cylinder\",\"allOf\":[{\"$ref\":\"#/definitions/common"
    "\"},{\"required\":[\"shape\",\"diameter\",\"height\"],\"type\":\"object\",\"descript"
    "ion\":\"cylinder specific properties\",\"properties\":{\"diameter\":{\"type\":\""
    "number\",\"description\":\"diameter in meters\",\"exclusiveMinimum\":true,\"mi"
    "nimum\":0},\"height\":{\"type\":\"number\",\"description\":\"height in meters\",\""
    "exclusiveMinimum\":true,\"minimum\":0},\"shape\":{\"type\":\"string\",\"enum\":[\""
    "cylinder\"],\"description\":\"shape identifier\"}}}]},\"sphere\":{\"type\":\"obj"
    "ect\",\"description\":\"sphere\",\"allOf\":[{\"$ref\":\"#/definitions/common\"},{"
    "\"required\":[\"shape\",\"diameter\"],\"type\":\"object\",\"description\":\"sphere "
    "specific properties\",\"properties\":{\"diameter\":{\"type\":\"number\",\"descri"
    "ption\":\"diameter in meters\",\"exclusiveMinimum\":true,\"minimum\":0},\"shap"
    "e\":{\"type\":\"string\",\"enum\":[\"sphere\"],\"description\":\"shape identifier\""
    "}}}]},\"circular_disk\":{\"type\":\"object\",\"description\":\"circular disk\",\""
    "allOf\":[{\"$ref\":\"#/definitions/common\"},{\"required\":[\"shape\",\"diameter"
    "\"],\"type\":\"object\",\"description\":\"circular disk specific properties\",\""
    "properties\":{\"diameter\":{\"type\":\"number\",\"description\":\"diameter in me"
    "ters\",\"exclusiveMinimum\":true,\"minimum\":0},\"shape\":{\"type\":\"string\",\"e"
    "num\":[\"circular_disk\"],\"description\":\"shape "
    "identifier\"},\"offset\":{\"type\":\"number\",\"description\":\"offset in "
    "meters, e.g. reflector constant (optional)\"}}}]},\"rectangle\":{\"type\":\""
    "object\",\"description\":\"rectangle\",\"allOf\":[{\"$ref\":\"#/definitions/comm"
    "on\"},{\"required\":[\"shape\",\"length\",\"width\"],\"type\":\"object\",\"descripti"
    "on\":\"rectangle specific properties\",\"properties\":{\"length\":{\"type\":\"nu"
    "mber\",\"description\":\"length in meters\",\"exclusiveMinimum\":true,\"minimu"
    "m\":0},\"width\":{\"type\":\"number\",\"description\":\"width in meters\",\"exclus"
    "iveMinimum\":true,\"minimum\":0},\"shape\":{\"type\":\"string\",\"enum\":[\"rectan"
    "gle\"]}}}]},\"common\":{\"required\":[\"name\",\"shape\"],\"type\":\"object\",\"desc"
    "ription\":\"common object properties\",\"properties\":{\"description\":{\"type"
    "\":\"string\",\"description\":\"string describing the object\"},\"surface_type"
    "\":{\"type\":\"string\",\"enum\":[\"retro_reflective_foil\",\"diffuse\"],\"descrip"
    "tion\":\"surface material "
    "type\"},\"name\":{\"type\":\"string\",\"description\":\"unique type identifier\","
    "\"minLength\":3},\"shape\":{\"type\":\"string\",\"enum\":[\"rectangle\",\"checkerbo"
    "ard2x2\",\"chevron\",\"circular_disk\",\"cylinder\",\"sphere\",\"round_corner_cu"
    "be_prism\"],\"description\":\"shape identifier\"}}},\"checkerboard2x2\":{\"typ"
    "e\":\"object\",\"description\":\"checkerboard 2 by 2\",\"allOf\":[{\"$ref\":\"#/de"
    "finitions/common\"},{\"required\":[\"shape\",\"square_length\"],\"type\":\"objec"
    "t\",\"description\":\"checkerboard specific properties\",\"properties\":{\"squ"
    "are_length\":{\"type\":\"number\",\"description\":\"length of a square of the "
    "checkerboard in meters\",\"exclusiveMinimum\":true,\"minimum\":0},\"shape\":{"
    "\"type\":\"string\",\"enum\":[\"checkerboard2x2\"],\"description\":\"shape identi"
    "fier\"}}}]},\"round_corner_cube_prism\":{\"type\":\"object\",\"description\":\"r"
    "ound corner cube prism\",\"allOf\":[{\"$ref\":\"#/definitions/common\"},{\"req"
    "uired\":[\"shape\",\"diameter\"],\"type\":\"object\",\"description\":\"round "
    "corner cube prism specific properties\",\"properties\":{\"diameter\":{\"type"
    "\":\"number\",\"description\":\"diameter in meters\",\"exclusiveMinimum\":true,"
    "\"minimum\":0},\"shape\":{\"type\":\"string\",\"enum\":[\"round_corner_cube_prism"
    "\"],\"description\":\"shape "
    "identifier\"},\"offset\":{\"type\":\"number\",\"description\":\"offset in "
    "meters, e.g. reflector constant (optional)\"}}}]}},\"properties\":{\"types"
    "\":{\"type\":\"array\",\"items\":{\"type\":\"object\",\"oneOf\":[{\"$ref\":\"#/definit"
    "ions/rectangle\"},{\"$ref\":\"#/definitions/checkerboard2x2\"},{\"$ref\":\"#/d"
    "efinitions/chevron\"},{\"$ref\":\"#/definitions/circular_disk\"},{\"$ref\":\"#"
    "/definitions/cylinder\"},{\"$ref\":\"#/definitions/sphere\"},{\"$ref\":\"#/def"
    "initions/round_corner_cube_prism\"}]},\"uniqueItems\":true}}}"
    ,
    /* example: */
    "{\"types\":[{\"length\":0.6,\"description\":\"Rectangle (60cm x "
    "30cm)\",\"width\":0.3,\"name\":\"Rectangle "
    "60x30\",\"shape\":\"rectangle\"},{\"length\":0.8,\"description\":\"Rectangle "
    "(80cm x 40cm)\",\"width\":0.4,\"name\":\"Rectangle 80x40\",\"shape\":\"rectangle"
    "\"},{\"square_length\":0.3,\"description\":\"Checkerboard (square length: "
    "30cm)\",\"name\":\"Checkerboard2x2 30\",\"shape\":\"checkerboard2x2\"},{\"square"
    "_length\":0.5,\"description\":\"Checkerboard (square length: "
    "50cm)\",\"name\":\"Checkerboard2x2 50\",\"shape\":\"checkerboard2x2\"},{\"outsid"
    "e_edge_length\":0.6096,\"description\":\"Chevron (a=24''; "
    "b=4'')\",\"name\":\"Chevron 24''/4''\",\"shape\":\"chevron\",\"thickness\":0.1016"
    "},{\"surface_type\":\"diffuse\",\"description\":\" Circular Disk (diameter: "
    "50cm)\",\"diameter\":0.5,\"name\":\"Circular Disk 50\",\"shape\":\"circular_disk"
    "\"},{\"surface_type\":\"retro_reflective_foil\",\"description\":\"flat "
    "circular reflector from retro-reflective "
    "foil\",\"diameter\":0.05,\"name\":\"RIEGL flat reflector 50 mm\",\"shape\":\"cir"
    "cular_disk\",\"offset\":0.0},{\"surface_type\":\"retro_reflective_foil\",\"des"
    "cription\":\"flat circular reflector from retro-reflective "
    "foil\",\"diameter\":0.1,\"name\":\"RIEGL flat reflector 100 mm\",\"shape\":\"cir"
    "cular_disk\",\"offset\":0.0},{\"surface_type\":\"retro_reflective_foil\",\"des"
    "cription\":\"flat circular reflector from retro-reflective "
    "foil\",\"diameter\":0.15,\"name\":\"RIEGL flat reflector 150 mm\",\"shape\":\"ci"
    "rcular_disk\",\"offset\":0.0},{\"surface_type\":\"retro_reflective_foil\",\"sh"
    "ape\":\"cylinder\",\"description\":\"cylindrical reflector from "
    "retro-reflective foil\",\"diameter\":0.05,\"name\":\"RIEGL cylindrical "
    "reflector 50 mm\",\"height\":0.05},{\"surface_type\":\"retro_reflective_foil"
    "\",\"shape\":\"cylinder\",\"description\":\"cylindrical reflector from "
    "retro-reflective foil\",\"diameter\":0.1,\"name\":\"RIEGL cylindrical "
    "reflector 100 mm\",\"height\":0.1},{\"description\":\"Sphere (diameter: 200 "
    "mm)\",\"diameter\":0.2,\"name\":\"Sphere 200 "
    "mm\",\"shape\":\"sphere\"},{\"description\":\"round corner cube "
    "prism\",\"diameter\":0.05,\"name\":\"Corner Cube Prism 50 "
    "mm\",\"shape\":\"round_corner_cube_prism\",\"offset\":0.0}],\"comments\":[\"This"
    " file contains a list of control object types (aka. 'catalog').\",\"Each"
    " type is described by an object,\",\"which must contain at least the "
    "following parameters:\",\"  - name: unique identifier of the type\",\"  - "
    "shape: one of the following supported shapes:\",\"      - rectangle\",\""
    "      - checkerboard2x2\",\"      - chevron\",\"      - circular_disk\",\""
    "      - cylinder\",\"      - sphere\",\"      - "
    "round_corner_cube_prism\",\"Depending on 'shape', the following "
    "parameters must/may be specified:\",\"  - rectangle:\",\"      - length: "
    "length in meters\",\"      - width: width in meters\",\"  - "
    "checkerboard2x2:\",\"      - square_length: length of a square of the "
    "checkerboard in meters\",\"  - circular_disk:\",\"      - diameter: "
    "diameter in meters\",\"      - offset: offset in meters, e.g. reflector "
    "constant (optional)\",\"  - chevron:\",\"      - outside_edge_length: "
    "length of the two outer edges in meters\",\"      - thickness: thickness"
    " in meters\",\"  - cylinder:\",\"      - diameter: diameter in meters\",\""
    "      - height:  height in meters\",\"  - sphere:\",\"      - diameter: "
    "diameter in meters\",\"  - round_corner_cube_prism:\",\"      - diameter: "
    "diameter in meters\",\"      - offset: offset in meters, e.g. reflector "
    "constant (optional)\",\"Optional parameters:\",\"    - description: string"
    " describing the object\",\"    - surface_type: surface material type "
    "(either 'retro_reflective_foil' or 'diffuse')\"]}"
};

// Details about the control object reference file
const MetaDataItemInfo RDB_RIEGL_CONTROL_OBJECT_REFERENCE_FILE = {
    /* name        */ "riegl.control_object_reference_file",
    /* title       */ "Control Object Reference file",
    /* description */ "Details about the control object reference file",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"properties\":{\"reference_file\":{\"required\":[\"file_uuid\",\"file_path\"],"
    "\"type\":\"object\",\"description\":\"Reference to a control object file\",\"pr"
    "operties\":{\"file_uuid\":{\"type\":\"string\",\"description\":\"Control object "
    "file's Universally Unique Identifier (RFC "
    "4122)\"},\"file_path\":{\"type\":\"string\",\"description\":\"Path of the "
    "control object file relative to referring file\"}}}},\"type\":\"object\",\"$"
    "schema\":\"http://json-schema.org/draft-04/schema#\",\"description\":\"Detai"
    "ls about the control object reference file\",\"title\":\"Control Object "
    "Reference file\"}"
    ,
    /* example: */
    "{\"reference_file\":{\"file_uuid\":\"810f5d57-eccf-49ed-b07a-0cdd109b4213\","
    "\"file_path\":\"../../../10_CONTROL_OBJECTS/ControlPoints.cpx\"}}"
};

// Detection probability as a function of amplitude
const MetaDataItemInfo RDB_RIEGL_DETECTION_PROBABILITY = {
    /* name        */ "riegl.detection_probability",
    /* title       */ "Detection Probability",
    /* description */ "Detection probability as a function of amplitude",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"required\":[\"amplitude\",\"detection_probability\"],\"type\":\"object\",\"$sc"
    "hema\":\"http://json-schema.org/draft-04/schema#\",\"title\":\"Detection "
    "Probability\",\"description\":\"Detection probability as a function of amp"
    "litude\",\"properties\":{\"detection_probability\":{\"type\":\"array\",\"descrip"
    "tion\":\"Detection probability [0..1]\",\"items\":{\"type\":\"number\"}},\"ampli"
    "tude\":{\"type\":\"array\",\"description\":\"Amplitude "
    "[dB]\",\"items\":{\"type\":\"number\"}}}}"
    ,
    /* example: */
    "{\"detection_probability\":[0.0,0.5,0.8,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]"
    ",\"amplitude\":[0.0,1.0,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0,10.0]}"
};

// Details about the device used to acquire the point cloud
const MetaDataItemInfo RDB_RIEGL_DEVICE = {
    /* name        */ "riegl.device",
    /* title       */ "Device Information",
    /* description */ "Details about the device used to acquire the point cloud",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"required\":[\"device_type\",\"serial_number\"],\"type\":\"object\",\"$schema\":"
    "\"http://json-schema.org/draft-04/schema#\",\"title\":\"Device "
    "Information\",\"description\":\"Details about the device used to acquire "
    "the point cloud\",\"properties\":{\"channel_text\":{\"type\":\"string\",\"descri"
    "ption\":\"Optional channel description (e.g. 'Green Channel' for "
    "multi-channel "
    "devices)\"},\"serial_number\":{\"type\":\"string\",\"description\":\"Device "
    "serial number (e.g. "
    "S2221234)\"},\"device_build\":{\"type\":\"string\",\"description\":\"Device "
    "build "
    "variant\"},\"channel_number\":{\"type\":\"integer\",\"description\":\"Laser "
    "channel number (not defined or 0: single channel device)\",\"exclusiveMi"
    "nimum\":false,\"minimum\":0},\"device_type\":{\"type\":\"string\",\"description\""
    ":\"Device type identifier (e.g. "
    "VZ-400i)\"},\"device_name\":{\"type\":\"string\",\"description\":\"Optional "
    "device name (e.g. 'Scanner 1' for multi-scanner systems)\"}}}"
    ,
    /* example: */
    "{\"channel_text\":\"\",\"serial_number\":\"S2221234\",\"device_build\":\"\",\"chann"
    "el_number\":0,\"device_type\":\"VZ-400i\",\"device_name\":\"Scanner 1\"}"
};

// Scanner device geometry details
const MetaDataItemInfo RDB_RIEGL_DEVICE_GEOMETRY = {
    /* name        */ "riegl.device_geometry",
    /* title       */ "Device Geometry",
    /* description */ "Scanner device geometry details",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"required\":[\"primary\"],\"type\":\"object\",\"$schema\":\"http://json-schema."
    "org/draft-04/schema#\",\"title\":\"Device Geometry\",\"description\":\"Scanner"
    " device geometry "
    "details\",\"properties\":{\"amu\":{\"type\":\"object\",\"description\":\"Angle "
    "Measurement "
    "Unit\",\"properties\":{\"frameCC\":{\"type\":\"number\",\"description\":\"Frame "
    "Circle Count (number of LSBs per full rotation about frame axis)\",\"exc"
    "lusiveMinimum\":false,\"minimum\":0},\"lineCC\":{\"type\":\"number\",\"descripti"
    "on\":\"Line Circle Count (number of LSBs per full rotation about line ax"
    "is)\",\"exclusiveMinimum\":false,\"minimum\":0}}},\"primary\":{\"required\":[\"I"
    "D\",\"content\"],\"type\":\"object\",\"description\":\"Primary device geometry "
    "structure (mandatory)\",\"properties\":{\"ID\":{\"minItems\":2,\"type\":\"array\""
    ",\"description\":\"Structure identifier\",\"items\":{\"type\":\"integer\"},\"maxI"
    "tems\":2},\"content\":{\"type\":\"array\",\"description\":\"Internal calibration"
    " values\",\"items\":{\"type\":\"number\"}}}},\"secondary\":{\"required\":[\"ID\",\"c"
    "ontent\"],\"type\":\"object\",\"description\":\"Additional device geometry "
    "structure (optional)\",\"properties\":{\"ID\":{\"minItems\":2,\"type\":\"array\","
    "\"description\":\"Structure identifier\",\"items\":{\"type\":\"integer\"},\"maxIt"
    "ems\":2},\"content\":{\"type\":\"array\",\"description\":\"Internal calibration "
    "values\",\"items\":{\"type\":\"number\"}}}}}}"
    ,
    /* example: */
    "{\"amu\":{\"frameCC\":124000,\"lineCC\":124000},\"primary\":{\"ID\":[4,0],\"conte"
    "nt\":[0]},\"secondary\":{\"ID\":[91,0],\"content\":[0]}}"
};

// Scanner passive channel device geometry details
const MetaDataItemInfo RDB_RIEGL_DEVICE_GEOMETRY_PASSIVE_CHANNEL = {
    /* name        */ "riegl.device_geometry_passive_channel",
    /* title       */ "Device Geometry Passive Channel",
    /* description */ "Scanner passive channel device geometry details",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"required\":[\"primary\"],\"type\":\"object\",\"$schema\":\"http://json-schema."
    "org/draft-04/schema#\",\"title\":\"Device Geometry Passive "
    "Channel\",\"description\":\"Scanner passive channel device geometry detail"
    "s\",\"properties\":{\"primary\":{\"required\":[\"ID\",\"content\"],\"type\":\"object"
    "\",\"description\":\"Primary device geometry structure (mandatory)\",\"prope"
    "rties\":{\"ID\":{\"minItems\":2,\"type\":\"array\",\"description\":\"Structure ide"
    "ntifier\",\"items\":{\"type\":\"integer\"},\"maxItems\":2},\"content\":{\"type\":\"a"
    "rray\",\"description\":\"Internal calibration "
    "values\",\"items\":{\"type\":\"number\"}}}}}}"
    ,
    /* example: */
    "{\"primary\":{\"ID\":[143,0],\"content\":[0]}}"
};

// Limits of the measured values output by the device
const MetaDataItemInfo RDB_RIEGL_DEVICE_OUTPUT_LIMITS = {
    /* name        */ "riegl.device_output_limits",
    /* title       */ "Device Output Limits",
    /* description */ "Limits of the measured values output by the device",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"properties\":{\"deviation_minimum\":{\"type\":\"number\",\"description\":\"Min"
    "imum possible pulse shape deviation.\"},\"background_radiation_maximum\":"
    "{\"type\":\"number\",\"description\":\"Maximum possible background radiation."
    "\"},\"amplitude_minimum\":{\"type\":\"number\",\"description\":\"Minimum "
    "possible amplitude in "
    "dB.\"},\"deviation_maximum\":{\"type\":\"number\",\"description\":\"Maximum "
    "possible pulse shape deviation.\"},\"amplitude_maximum\":{\"type\":\"number\""
    ",\"description\":\"Maximum possible amplitude in "
    "dB.\"},\"reflectance_minimum\":{\"type\":\"number\",\"description\":\"Minimum "
    "possible reflectance in "
    "dB.\"},\"reflectance_maximum\":{\"type\":\"number\",\"description\":\"Maximum "
    "possible reflectance in "
    "dB.\"},\"mta_zone_count_maximum\":{\"type\":\"number\",\"description\":\"Maximum"
    " number of MTA "
    "zones.\"},\"range_minimum\":{\"type\":\"number\",\"description\":\"Minimum "
    "possible range in "
    "meters.\"},\"echo_count_maximum\":{\"type\":\"number\",\"description\":\"Maximum"
    " number of echoes a laser shot can have.\"},\"background_radiation_minim"
    "um\":{\"type\":\"number\",\"description\":\"Minimum possible background "
    "radiation.\"},\"range_maximum\":{\"type\":\"number\",\"description\":\"Maximum "
    "possible range in meters.\"}},\"type\":\"object\",\"$schema\":\"http://json-sc"
    "hema.org/draft-04/schema#\",\"description\":\"Limits of the measured "
    "values output by the device. The limits depend on the device type, "
    "measurement program and/or scan pattern.\",\"title\":\"Device Output "
    "Limits\"}"
    ,
    /* example: */
    "{\"deviation_minimum\":-1,\"background_radiation_maximum\":0,\"amplitude_mi"
    "nimum\":0.0,\"range_minimum\":2.9,\"deviation_maximum\":32767,\"amplitude_ma"
    "ximum\":100.0,\"reflectance_minimum\":-100.0,\"mta_zone_count_maximum\":7,\""
    "background_radiation_minimum\":0,\"reflectance_maximum\":100.0,\"range_max"
    "imum\":10000.0}"
};

// Details about the devices used to acquire the point cloud
const MetaDataItemInfo RDB_RIEGL_DEVICES = {
    /* name        */ "riegl.devices",
    /* title       */ "Devices Information",
    /* description */ "Details about the devices used to acquire the point cloud",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"type\":\"array\",\"items\":{\"type\":\"object\",\"required\":[\"device_type\",\"se"
    "rial_number\"],\"properties\":{\"channel_text\":{\"type\":\"string\",\"descripti"
    "on\":\"Optional channel description (e.g. 'Green Channel' for "
    "multi-channel "
    "devices)\"},\"serial_number\":{\"type\":\"string\",\"description\":\"Device "
    "serial number (e.g. "
    "S2221234)\"},\"device_build\":{\"type\":\"string\",\"description\":\"Device "
    "build "
    "variant\"},\"channel_number\":{\"type\":\"integer\",\"description\":\"Laser "
    "channel number (not defined or 0: single channel device)\",\"exclusiveMi"
    "nimum\":false,\"minimum\":0},\"device_type\":{\"type\":\"string\",\"description\""
    ":\"Device type identifier (e.g. "
    "VZ-400i)\"},\"signed\":{\"type\":\"boolean\",\"description\":\"Flag that is set "
    "when the original 'riegl.device' entry in the source file was "
    "correctly "
    "signed.\"},\"device_name\":{\"type\":\"string\",\"description\":\"Optional "
    "device name (e.g. 'Scanner 1' for multi-scanner systems)\"}}},\"$schema\""
    ":\"http://json-schema.org/draft-04/schema#\",\"description\":\"Details "
    "about the devices used to acquire the point cloud (e.g. when merging "
    "point clouds of different devices)\",\"title\":\"Devices Information\"}"
    ,
    /* example: */
    "[{\"channel_text\":\"\",\"serial_number\":\"S2221234\",\"device_build\":\"\",\"chan"
    "nel_number\":0,\"device_type\":\"VZ-400i\",\"signed\":false,\"device_name\":\"Sc"
    "anner 1\"},{\"channel_text\":\"\",\"serial_number\":\"S2222680\",\"device_build\""
    ":\"\",\"channel_number\":1,\"device_type\":\"VQ-1560i-DW\",\"signed\":true,\"devi"
    "ce_name\":\"Scanner 2\"},{\"channel_text\":\"\",\"serial_number\":\"S2222680\",\"d"
    "evice_build\":\"\",\"channel_number\":2,\"device_type\":\"VQ-1560i-DW\",\"signed"
    "\":true,\"device_name\":\"Scanner 3\"}]"
};

// Details about echo files
const MetaDataItemInfo RDB_RIEGL_ECHO_INFO = {
    /* name        */ "riegl.echo_info",
    /* title       */ "Echo Information",
    /* description */ "Details about echo files",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"required\":[\"echo_file\"],\"type\":\"object\",\"$schema\":\"http://json-schem"
    "a.org/draft-04/schema#\",\"title\":\"Echo "
    "Information\",\"description\":\"Details about echo files\",\"properties\":{\"e"
    "cho_file\":{\"type\":\"object\",\"required\":[\"file_extension\"],\"properties\":"
    "{\"file_uuid\":{\"type\":\"string\",\"description\":\"File's Universally Unique"
    " Identifier (RFC "
    "4122)\"},\"file_extension\":{\"type\":\"string\",\"description\":\"Echo file "
    "extension, without the leading dot\"}}}}}"
    ,
    /* example: */
    "{\"echo_file\":{\"file_uuid\":\"26a03615-67c0-4bea-8fe8-c577378fe661\",\"file"
    "_extension\":\"owp\"}}"
};

// Details for exponential decomposition of full waveform data
const MetaDataItemInfo RDB_RIEGL_EXPONENTIAL_DECOMPOSITION = {
    /* name        */ "riegl.exponential_decomposition",
    /* title       */ "Exponential Decomposition",
    /* description */ "Details for exponential decomposition of full waveform data",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"type\":\"object\",\"patternProperties\":{\"^[0-9]+$\":{\"description\":\"one "
    "field per channel, field name is channel index\",\"$ref\":\"#/definitions/"
    "channel\"}},\"$schema\":\"http://json-schema.org/draft-04/schema#\",\"title\""
    ":\"Exponential Decomposition\",\"description\":\"Details for exponential "
    "decomposition of full waveform data\",\"definitions\":{\"channel\":{\"type\":"
    "\"object\",\"required\":[\"delay\",\"scale\",\"parameter\"],\"properties\":{\"a_lin"
    "\":{\"type\":\"number\",\"exclusiveMaximum\":false,\"minimum\":0,\"maximum\":1,\"d"
    "escription\":\"relative linear amplitude range [0..1]\",\"exclusiveMinimum"
    "\":false},\"delay\":{\"type\":\"number\",\"description\":\"delay calibration in "
    "seconds\"},\"parameter\":{\"required\":[\"A\",\"B\",\"gamma\",\"omega\"],\"type\":\"ob"
    "ject\",\"description\":\"parameters of the syswave exponential "
    "sum\",\"properties\":{\"omega\":{\"type\":\"array\",\"description\":\"angular "
    "frequency in "
    "Hz\",\"items\":{\"type\":\"number\"}},\"A\":{\"type\":\"array\",\"description\":\"real"
    " part of amplitude factor in units of full-scale\",\"items\":{\"type\":\"num"
    "ber\"}},\"gamma\":{\"type\":\"array\",\"description\":\"decay in 1/second\",\"item"
    "s\":{\"type\":\"number\"}},\"B\":{\"type\":\"array\",\"description\":\"imaginary "
    "part of amplitude factor in units of full-scale\",\"items\":{\"type\":\"numb"
    "er\"}}}},\"scale\":{\"type\":\"number\",\"description\":\"amplitude "
    "calibration\"}}}},\"additionalProperties\":false}"
    ,
    /* example: */
    "{\"0\":{\"a_lin\":0.27,\"delay\":3.783458418887631e-09,\"parameter\":{\"omega\":"
    "[352020896.0,3647927552.0,-1977987072.0],\"A\":[0.9772450923919678,0.335"
    "4335129261017,-1.312678575515747],\"gamma\":[-1094726528.0,-769562752.0,"
    "-848000064.0],\"B\":[-3.9813032150268555,0.08622030913829803,-0.31528609"
    "99107361]},\"scale\":1.0},\"1\":{\"a_lin\":0.9,\"delay\":3.5e-09,\"parameter\":{"
    "\"omega\":[352020896.0,3647927552.0,-1977987072.0],\"A\":[0.9,0.3,-1.3],\"g"
    "amma\":[-1094726528.0,-769562752.0,-848000064.0],\"B\":[-3.9,0.0,-0.3]},\""
    "scale\":1.0}}"
};

// Details for Gaussian decomposition of full waveform data
const MetaDataItemInfo RDB_RIEGL_GAUSSIAN_DECOMPOSITION = {
    /* name        */ "riegl.gaussian_decomposition",
    /* title       */ "Gaussian Decomposition",
    /* description */ "Details for Gaussian decomposition of full waveform data",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"required\":[\"amplitude_lsb_low_power\",\"amplitude_lsb_high_power\",\"amp"
    "litude_db\",\"range_offset_sec_low_power\",\"range_offset_sec_high_power\"]"
    ",\"type\":\"object\",\"$schema\":\"http://json-schema.org/draft-04/schema#\",\""
    "title\":\"Gaussian "
    "Decomposition\",\"description\":\"riegl.gaussian_decomposition contains "
    "information relevant for extracting calibrated amplitudes and "
    "calibrated ranges from a Gaussian decomposition of full waveform data."
    " This information is contained in a table with five columns. Two "
    "columns are to be used as input: amplitude_lsb_low_power and "
    "amplitude_lsb_high_power. The other three columns provide the outputs."
    " Amplitude_db gives the calibrated amplitude in the optical regime in "
    "decibels. The range offset columns provide additive range offsets, "
    "given in units of seconds, for each channel.\",\"properties\":{\"range_off"
    "set_sec_low_power\":{\"type\":\"array\",\"items\":{\"type\":\"number\"}},\"range_o"
    "ffset_sec_high_power\":{\"type\":\"array\",\"items\":{\"type\":\"number\"}},\"ampl"
    "itude_db\":{\"type\":\"array\",\"items\":{\"type\":\"number\"}},\"amplitude_lsb_lo"
    "w_power\":{\"type\":\"array\",\"items\":{\"type\":\"number\"}},\"amplitude_lsb_hig"
    "h_power\":{\"type\":\"array\",\"items\":{\"type\":\"number\"}}}}"
    ,
    /* example: */
    "{\"range_offset_sec_low_power\":[],\"range_offset_sec_high_power\":[],\"amp"
    "litude_db\":[],\"amplitude_lsb_low_power\":[],\"amplitude_lsb_high_power\":"
    "[]}"
};

// Point cloud georeferencing information
const MetaDataItemInfo RDB_RIEGL_GEO_TAG = {
    /* name        */ "riegl.geo_tag",
    /* title       */ "Geo Tag",
    /* description */ "Point cloud georeferencing information",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"properties\":{\"pose\":{\"minItems\":4,\"type\":\"array\",\"description\":\"Coor"
    "dinate Transformation Matrix to transform from File Coordinate System "
    "to Global Coordinate Reference System. 4x4 matrix stored as two "
    "dimensional array, row major order.\",\"items\":{\"minItems\":4,\"type\":\"arr"
    "ay\",\"description\":\"rows\",\"items\":{\"type\":\"number\",\"description\":\"colum"
    "ns\"},\"maxItems\":4},\"maxItems\":4},\"crs\":{\"type\":\"object\",\"description\":"
    "\"Global Coordinate Reference System. Please note that only 3D "
    "Cartesian Coordinate Systems are "
    "allowed.\",\"properties\":{\"epsg\":{\"type\":\"integer\",\"description\":\"EPSG "
    "code\",\"minimum\":0},\"wkt\":{\"type\":\"string\",\"description\":\"\\\"Well-Known "
    "Text\\\" string, OGC WKT dialect (see http://www.opengeospatial.org/stan"
    "dards/wkt-crs)\"},\"name\":{\"type\":\"string\",\"description\":\"Coordinate "
    "reference system name\"}}}},\"type\":\"object\",\"$schema\":\"http://json-sche"
    "ma.org/draft-04/schema#\",\"description\":\"Point cloud georeferencing "
    "information\",\"title\":\"Geo Tag\"}"
    ,
    /* example: */
    "{\"pose\":[[-0.269827776749716,-0.723017716139738,0.635954678449952,4063"
    "882.500831],[0.962908599449764,-0.20260517250352,0.178208229833847,113"
    "8787.607461],[0.0,0.660451759194338,0.7508684796801,4766084.550196],[0"
    ".0,0.0,0.0,1.0]],\"crs\":{\"epsg\":4978,\"wkt\":\"GEOCCS[\\\"WGS84 Geocentric\\\""
    ",DATUM[\\\"WGS84\\\",SPHEROID[\\\"WGS84\\\",6378137.000,298.257223563,AUTHORIT"
    "Y[\\\"EPSG\\\",\\\"7030\\\"]],AUTHORITY[\\\"EPSG\\\",\\\"6326\\\"]],PRIMEM[\\\"Greenwich"
    "\\\",0.0000000000000000,AUTHORITY[\\\"EPSG\\\",\\\"8901\\\"]],UNIT[\\\"Meter\\\",1.0"
    "0000000000000000000,AUTHORITY[\\\"EPSG\\\",\\\"9001\\\"]],AXIS[\\\"X\\\",OTHER],AX"
    "IS[\\\"Y\\\",EAST],AXIS[\\\"Z\\\",NORTH],AUTHORITY[\\\"EPSG\\\",\\\"4978\\\"]]\",\"name\""
    ":\"WGS84 Geocentric\"}}"
};

// Geometric scale factor applied to point coordinates
const MetaDataItemInfo RDB_RIEGL_GEOMETRIC_SCALE_FACTOR = {
    /* name        */ "riegl.geometric_scale_factor",
    /* title       */ "Geometric Scale Factor",
    /* description */ "Geometric scale factor applied to point coordinates",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"type\":\"number\",\"exclusiveMinimum\":true,\"$schema\":\"http://json-schema"
    ".org/draft-04/schema#\",\"description\":\"Geometric scale factor applied "
    "to point coordinates\",\"minimum\":0}"
    ,
    /* example: */
    "1.0"
};

// Parameters used for georeferencing of the point cloud
const MetaDataItemInfo RDB_RIEGL_GEOREFERENCING_PARAMETERS = {
    /* name        */ "riegl.georeferencing_parameters",
    /* title       */ "Georeferencing Parameters",
    /* description */ "Parameters used for georeferencing of the point cloud",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"properties\":{\"socs_to_rocs_matrix\":{\"minItems\":4,\"type\":\"array\",\"des"
    "cription\":\"Coordinate Transformation Matrix to transform from "
    "Scanner's Own Coordinate System to Record Coordinate System. 4x4 "
    "matrix stored as two dimensional array, row major order.\",\"items\":{\"mi"
    "nItems\":4,\"type\":\"array\",\"description\":\"rows\",\"items\":{\"type\":\"number\""
    ",\"description\":\"columns\"},\"maxItems\":4},\"maxItems\":4},\"body_coordinate"
    "_system_type\":{\"type\":\"string\",\"enum\":[\"NED\",\"ENU\"],\"description\":\"BOD"
    "Y coordinate frame (NED: North-East-Down, ENU: East-North-Up), "
    "default: "
    "NED\"},\"trajectory_offsets\":{\"type\":\"object\",\"description\":\"Correction "
    "offsets applied to the trajectory data\",\"properties\":{\"offset_north\":{"
    "\"type\":\"number\",\"description\":\"[m]\"},\"offset_east\":{\"type\":\"number\",\"d"
    "escription\":\"[m]\"},\"offset_height\":{\"type\":\"number\",\"description\":\"[m]"
    "\"},\"offset_time\":{\"type\":\"number\",\"description\":\"[s]\"},\"offset_yaw\":{\""
    "type\":\"number\",\"description\":\"[deg]\"},\"version\":{\"type\":\"integer\",\"des"
    "cription\":\"Meaning of offset values and how to apply them; version 0: "
    "Rz(yaw+offset_yaw)*Ry(pitch+offset_pitch)*Rx(roll+offset_roll), "
    "version 1: Rz(yaw)*Ry(pitch)*Rx(roll) * Rz(offset_yaw)*Ry(offset_pitch"
    ")*Rx(offset_roll)\"},\"offset_roll\":{\"type\":\"number\",\"description\":\"[deg"
    "]\"},\"offset_pitch\":{\"type\":\"number\",\"description\":\"[deg]\"}}},\"socs_to_"
    "body_matrix\":{\"minItems\":4,\"type\":\"array\",\"description\":\"Coordinate "
    "Transformation Matrix to transform from Scanner's Own Coordinate "
    "System to Body Coordinate System. 4x4 matrix stored as two dimensional"
    " array, row major order.\",\"items\":{\"minItems\":4,\"type\":\"array\",\"descri"
    "ption\":\"rows\",\"items\":{\"type\":\"number\",\"description\":\"columns\"},\"maxIt"
    "ems\":4},\"maxItems\":4},\"trajectory_file\":{\"required\":[\"file_extension\"]"
    ",\"type\":\"object\",\"description\":\"Trajectory data used for "
    "georeferencing of the point cloud\",\"properties\":{\"file_uuid\":{\"type\":\""
    "string\",\"description\":\"File's Universally Unique Identifier (RFC "
    "4122)\"},\"file_extension\":{\"type\":\"string\",\"description\":\"Trajectory "
    "file extension, without the leading dot\"}}}},\"type\":\"object\",\"$schema\""
    ":\"http://json-schema.org/draft-04/schema#\",\"description\":\"Parameters "
    "used for georeferencing of the point cloud\",\"title\":\"Georeferencing "
    "Parameters\"}"
    ,
    /* example: */
    "{\"body_coordinate_system_type\":\"NED\",\"trajectory_offsets\":{\"offset_nor"
    "th\":0.07,\"offset_east\":0.15,\"offset_height\":-0.2,\"offset_time\":18.007,"
    "\"offset_yaw\":-0.45,\"version\":0,\"offset_roll\":0.03,\"offset_pitch\":0.01}"
    ",\"socs_to_body_matrix\":[[-0.269827776749716,-0.723017716139738,0.63595"
    "4678449952,0.0],[0.962908599449764,-0.20260517250352,0.178208229833847"
    ",0.0],[0.0,0.660451759194338,0.7508684796801,0.0],[0.0,0.0,0.0,1.0]],\""
    "trajectory_file\":{\"file_uuid\":\"93a03615-66c0-4bea-8ff8-c577378fe660\",\""
    "file_extension\":\"pofx\"}}"
};

// Map of item names
const MetaDataItemInfo RDB_RIEGL_ITEM_NAMES = {
    /* name        */ "riegl.item_names",
    /* title       */ "Item Names",
    /* description */ "Map of item names",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"type\":\"object\",\"patternProperties\":{\"^-?[0-9]+$\":{\"type\":\"string\",\"d"
    "escription\":\"One field per item, field name is item id, field value is"
    " item name\"}},\"$schema\":\"http://json-schema.org/draft-04/schema#\",\"tit"
    "le\":\"Item Names\",\"description\":\"Map of item "
    "names\",\"additionalProperties\":false}"
    ,
    /* example: */
    "{\"2\":\"Name of item with id 2\",\"1\":\"Name of item with id 1\",\"47\":\"Name "
    "of item with id 47\",\"-1\":\"Name of item with id -1\"}"
};

// License keys for software features
const MetaDataItemInfo RDB_RIEGL_LICENSES = {
    /* name        */ "riegl.licenses",
    /* title       */ "Software License Keys",
    /* description */ "License keys for software features",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"type\":\"object\",\"patternProperties\":{\"^.*$\":{\"minItems\":1,\"type\":\"arr"
    "ay\",\"description\":\"Each field of the object represents a feature and "
    "holds a list of license keys, where the field name is the feature "
    "name.\",\"items\":{\"type\":\"string\",\"description\":\"License key (example: "
    "'46AE032A - 39882AC4 - 9EC0A184 - 6F163D73')\"}}},\"$schema\":\"http://jso"
    "n-schema.org/draft-04/schema#\",\"title\":\"Software License "
    "Keys\",\"description\":\"License keys for software "
    "features\",\"additionalProperties\":false}"
    ,
    /* example: */
    "{\"Georeferencing\":[\"46AE032A - 39882AC4 - 9EC0A184 - 6F163D73\"],\"MTA "
    "resolution\":[\"468E020A - 39A922E4 - B681A184 - 673E3D72\"],\"Full "
    "Waveform Analysis Topography\":[\"0FD5FF07 - 011A1255 - 9F76CACA - "
    "8D2ED557\"],\"Full Waveform Analysis Topography with GPU "
    "support\":[\"8AB44126 - 23B92250 - 16E2689F - 34EF7E7B\"]}"
};

// Parameters for MTA processing
const MetaDataItemInfo RDB_RIEGL_MTA_SETTINGS = {
    /* name        */ "riegl.mta_settings",
    /* title       */ "MTA Settings",
    /* description */ "Parameters for MTA processing",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"properties\":{\"zone_count\":{\"type\":\"integer\",\"maximum\":255,\"descripti"
    "on\":\"Maximum number of MTA zones.\",\"minimum\":0},\"modulation_depth\":{\"t"
    "ype\":\"number\",\"description\":\"Depth of pulse position modulation in met"
    "er.\",\"minimum\":0},\"zone_width\":{\"type\":\"number\",\"description\":\"Width "
    "of a MTA zone in meter.\",\"minimum\":0}},\"type\":\"object\",\"$schema\":\"http"
    "://json-schema.org/draft-04/schema#\",\"description\":\"Parameters for MTA"
    " processing\",\"title\":\"MTA Settings\"}"
    ,
    /* example: */
    "{\"zone_count\":23,\"modulation_depth\":9.368514,\"zone_width\":149.896225}"
};

// Lookup table for range correction based on raw range
const MetaDataItemInfo RDB_RIEGL_NEAR_RANGE_CORRECTION = {
    /* name        */ "riegl.near_range_correction",
    /* title       */ "Near Range Correction Table",
    /* description */ "Lookup table for range correction based on raw range",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"required\":[\"delta\",\"content\"],\"type\":\"object\",\"$schema\":\"http://json"
    "-schema.org/draft-04/schema#\",\"title\":\"Near Range Correction "
    "Table\",\"description\":\"Lookup table for range correction based on raw r"
    "ange\",\"properties\":{\"content\":{\"minItems\":1,\"type\":\"array\",\"descriptio"
    "n\":\"Correction value [m] to be added to the raw range\",\"items\":{\"type\""
    ":\"number\"},\"maxItems\":2000},\"delta\":{\"type\":\"number\",\"description\":\"De"
    "lta between table entries [m], first entry is at range = 0 m\"}}}"
    ,
    /* example: */
    "{\"content\":[0.0],\"delta\":0.512}"
};

// Standard deviation for range and amplitude as a function of amplitude
const MetaDataItemInfo RDB_RIEGL_NOISE_ESTIMATES = {
    /* name        */ "riegl.noise_estimates",
    /* title       */ "Noise Estimates",
    /* description */ "Standard deviation for range and amplitude as a function of amplitude",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"required\":[\"amplitude\",\"range_sigma\",\"amplitude_sigma\"],\"type\":\"obje"
    "ct\",\"$schema\":\"http://json-schema.org/draft-04/schema#\",\"title\":\"Noise"
    " Estimates\",\"description\":\"Standard deviation for range and amplitude "
    "as a function of amplitude\",\"properties\":{\"amplitude_sigma\":{\"type\":\"a"
    "rray\",\"description\":\"Sigma amplitude [dB]\",\"items\":{\"type\":\"number\"}},"
    "\"range_sigma\":{\"type\":\"array\",\"description\":\"Sigma range [m]\",\"items\":"
    "{\"type\":\"number\"}},\"amplitude\":{\"type\":\"array\",\"description\":\"Amplitud"
    "e [dB]\",\"items\":{\"type\":\"number\"}}}}"
    ,
    /* example: */
    "{\"amplitude_sigma\":[0.988,0.988,0.875,0.774,0.686,0.608,0.54,0.482,0.4"
    "32,0.39,0.354],\"range_sigma\":[0.065,0.056,0.046,0.038,0.032,0.027,0.02"
    "4,0.021,0.018,0.016,0.014],\"amplitude\":[0.0,1.0,2.0,3.0,4.0,5.0,6.0,7."
    "0,8.0,9.0,10.0]}"
};

// Notch filter parameters for window glass echoes
const MetaDataItemInfo RDB_RIEGL_NOTCH_FILTER = {
    /* name        */ "riegl.notch_filter",
    /* title       */ "Notch Filter",
    /* description */ "Notch filter parameters for window glass echoes",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"required\":[\"range_minimum\",\"range_maximum\",\"amplitude_maximum\"],\"typ"
    "e\":\"object\",\"$schema\":\"http://json-schema.org/draft-04/schema#\",\"title"
    "\":\"Notch Filter\",\"description\":\"Notch filter parameters for window "
    "glass echoes\",\"properties\":{\"amplitude_maximum\":{\"type\":\"number\",\"desc"
    "ription\":\"Maximum amplitude [dB]\",\"minimum\":0},\"range_minimum\":{\"type\""
    ":\"number\",\"description\":\"Minimum range "
    "[m]\"},\"range_maximum\":{\"type\":\"number\",\"description\":\"Maximum range "
    "[m]\"}}}"
    ,
    /* example: */
    "{\"amplitude_maximum\":18.0,\"range_minimum\":-0.5,\"range_maximum\":0.2}"
};

// Details about the pixels contained in the file
const MetaDataItemInfo RDB_RIEGL_PIXEL_INFO = {
    /* name        */ "riegl.pixel_info",
    /* title       */ "Pixel Information",
    /* description */ "Details about the pixels contained in the file",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"required\":[\"size\"],\"type\":\"object\",\"$schema\":\"http://json-schema.org"
    "/draft-04/schema#\",\"title\":\"Pixel Information\",\"description\":\"Details "
    "about the pixels contained in the file\",\"definitions\":{\"pixel_size\":{\""
    "minItems\":2,\"type\":\"array\",\"description\":\"Size of "
    "pixels.\",\"items\":{\"type\":\"number\",\"description\":\"Length of pixel edge "
    "[m].\",\"minimum\":0},\"maxItems\":2}},\"properties\":{\"size_llcs\":{\"descript"
    "ion\":\"Size of pixels in a locally levelled cartesian coordinate system"
    " (xy). This is only used for pixels based on a map projection.\",\"$ref\""
    ":\"#/definitions/pixel_size\"},\"size\":{\"description\":\"Size of pixels in "
    "file coordinate system.\",\"$ref\":\"#/definitions/pixel_size\"}}}"
    ,
    /* example: */
    "{\"size_llcs\":[0.5156575252891171,0.5130835356683303],\"size\":[0.5971642"
    "834779395,0.5971642834779395]}"
};

// Details about the plane patch matching process
const MetaDataItemInfo RDB_RIEGL_PLANE_PATCH_MATCHING = {
    /* name        */ "riegl.plane_patch_matching",
    /* title       */ "Plane Patch Matching",
    /* description */ "Details about the plane patch matching process",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"required\":[\"plane_patch_file_one\",\"plane_patch_file_two\"],\"type\":\"ob"
    "ject\",\"$schema\":\"http://json-schema.org/draft-04/schema#\",\"title\":\"Pla"
    "ne Patch Matching\",\"description\":\"Details about the plane patch "
    "matching process\",\"definitions\":{\"file_reference\":{\"required\":[\"file_u"
    "uid\",\"file_path\"],\"type\":\"object\",\"description\":\"Reference to a plane "
    "patch "
    "file\",\"properties\":{\"file_uuid\":{\"type\":\"string\",\"description\":\"Plane "
    "patch file's Universally Unique Identifier (RFC "
    "4122)\"},\"file_path\":{\"type\":\"string\",\"description\":\"Path of the plane "
    "patch file relative to the match file\"}}}},\"properties\":{\"plane_patch_"
    "file_two\":{\"description\":\"Reference to the plane patch file two\",\"$ref"
    "\":\"#/definitions/file_reference\"},\"plane_patch_file_one\":{\"description"
    "\":\"Reference to the plane patch file "
    "one\",\"$ref\":\"#/definitions/file_reference\"}}}"
    ,
    /* example: */
    "{\"plane_patch_file_two\":{\"file_uuid\":\"fa47d509-a64e-49ce-8b14-ff3130fb"
    "efa9\",\"file_path\":\"project.ptch\"},\"plane_patch_file_one\":{\"file_uuid\":"
    "\"810f5d57-eccf-49ed-b07a-0cdd109b4213\",\"file_path\":\"Record009_Line001/"
    "191025_121410_Scanner_1.ptch\"}}"
};

// Statistics about plane patches found by plane patch extractor
const MetaDataItemInfo RDB_RIEGL_PLANE_PATCH_STATISTICS = {
    /* name        */ "riegl.plane_patch_statistics",
    /* title       */ "Plane Patch Statistics",
    /* description */ "Statistics about plane patches found by plane patch extractor",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"properties\":{\"total_area\":{\"type\":\"number\",\"description\":\"sum of all"
    " plane patch areas [m\\u00b2]\"},\"total_horizontal_area\":{\"type\":\"number"
    "\",\"description\":\"sum of all plane patch areas projected to horizontal "
    "plane [m\\u00b2]\"}},\"type\":\"object\",\"$schema\":\"http://json-schema.org/d"
    "raft-04/schema#\",\"description\":\"Statistics about plane patches found "
    "by plane patch extractor\",\"title\":\"Plane Patch Statistics\"}"
    ,
    /* example: */
    "{\"total_area\":14007.965,\"total_horizontal_area\":13954.601}"
};

// Settings and classes for plane slope classification
const MetaDataItemInfo RDB_RIEGL_PLANE_SLOPE_CLASS_INFO = {
    /* name        */ "riegl.plane_slope_class_info",
    /* title       */ "Plane Slope Class Info",
    /* description */ "Settings and classes for plane slope classification",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"required\":[\"settings\",\"classes\"],\"type\":\"object\",\"$schema\":\"http://j"
    "son-schema.org/draft-04/schema#\",\"title\":\"Plane Slope Class "
    "Info\",\"description\":\"Settings and classes for plane slope classificati"
    "on\",\"definitions\":{\"method-1\":{\"required\":[\"plane_classification_metho"
    "d\",\"maximum_inclination_angle_horizontal\"],\"type\":\"object\",\"descriptio"
    "n\":\"Classification method 1\",\"properties\":{\"maximum_inclination_angle_"
    "horizontal\":{\"type\":\"number\",\"maximum\":360.0,\"description\":\"maximum "
    "inclination angle of horizontal plane patches [deg]\",\"minimum\":-360.0}"
    ",\"plane_classification_method\":{\"type\":\"integer\",\"maximum\":1,\"descript"
    "ion\":\"method ID (=1)\",\"minimum\":1}}},\"method-2\":{\"required\":[\"plane_cl"
    "assification_method\",\"sloping_plane_classes_minimum_angle\",\"sloping_pl"
    "ane_classes_maximum_angle\"],\"type\":\"object\",\"description\":\"Classificat"
    "ion method 2\",\"properties\":{\"sloping_plane_classes_maximum_angle\":{\"ty"
    "pe\":\"number\",\"maximum\":360.0,\"description\":\"maximum inclination angle "
    "of sloping plane patches [deg]\",\"minimum\":-360.0},\"sloping_plane_class"
    "es_minimum_angle\":{\"type\":\"number\",\"maximum\":360.0,\"description\":\"mini"
    "mum inclination angle of sloping plane patches [deg]\",\"minimum\":-360.0"
    "},\"plane_classification_method\":{\"type\":\"integer\",\"maximum\":2,\"descrip"
    "tion\":\"method ID (=2)\",\"minimum\":2}}}},\"properties\":{\"classes\":{\"type\""
    ":\"object\",\"patternProperties\":{\"^[0-9]+$\":{\"type\":\"string\",\"descriptio"
    "n\":\"one field per class, field name is class number, field value is "
    "class name\"}},\"description\":\"Class definition table\",\"additionalProper"
    "ties\":false},\"settings\":{\"type\":\"object\",\"description\":\"Classification"
    " settings, details see documentation of rdbplanes\",\"oneOf\":[{\"$ref\":\"#"
    "/definitions/method-1\"},{\"$ref\":\"#/definitions/method-2\"}]}}}"
    ,
    /* example: */
    "{\"classes\":{\"6\":\"vertical, pointing east\",\"2\":\"sloping, pointing up "
    "and east\",\"3\":\"sloping, pointing up and south\",\"13\":\"sloping, pointing"
    " down and west\",\"9\":\"vertical, pointing west\",\"1\":\"horizontal, "
    "pointing up\",\"10\":\"sloping, pointing down and east\",\"14\":\"horizontal, "
    "pointing down\",\"11\":\"sloping, pointing down and south\",\"7\":\"vertical, "
    "pointing south\",\"12\":\"sloping, pointing down and north\",\"4\":\"sloping, "
    "pointing up and north\",\"8\":\"vertical, pointing north\",\"5\":\"sloping, "
    "pointing up and west\"},\"settings\":{\"sloping_plane_classes_maximum_angl"
    "e\":70.0,\"sloping_plane_classes_minimum_angle\":10.0,\"plane_classificati"
    "on_method\":2}}"
};

// Grouping and sorting of point attributes for visualization purposes
const MetaDataItemInfo RDB_RIEGL_POINT_ATTRIBUTE_GROUPS = {
    /* name        */ "riegl.point_attribute_groups",
    /* title       */ "Point Attribute Groups",
    /* description */ "Grouping and sorting of point attributes for visualization purposes",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"type\":\"object\",\"patternProperties\":{\"^.*$\":{\"minItems\":1,\"type\":\"arr"
    "ay\",\"description\":\"Each field of the object represents a point "
    "attribute group and holds a list of point attributes, where the field "
    "name is the group name.\",\"items\":{\"type\":\"string\",\"description\":\"Point"
    " attribute full name or name pattern (perl regular expression syntax)\""
    "}}},\"$schema\":\"http://json-schema.org/draft-04/schema#\",\"title\":\"Point"
    " Attribute Groups\",\"description\":\"Grouping and sorting of point "
    "attributes for visualization purposes\",\"additionalProperties\":false}"
    ,
    /* example: */
    "{\"Other Attributes\":[\"riegl.selected\",\"riegl.visible\"],\"Primary Attrib"
    "utes\":[\"riegl.reflectance\",\"riegl.amplitude\",\"riegl.deviation\"],\"Time\""
    ":[\"riegl.timestamp\"],\"Coordinates/Vectors\":[\"riegl.xyz\",\"riegl.range\","
    "\"riegl.theta\",\"riegl.phi\"],\"Secondary "
    "Attributes\":[\"riegl.mirror_facet\",\"riegl.waveform_available\"]}"
};

// Details about point cloud files
const MetaDataItemInfo RDB_RIEGL_POINTCLOUD_INFO = {
    /* name        */ "riegl.pointcloud_info",
    /* title       */ "Point Cloud Information",
    /* description */ "Details about point cloud files",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"properties\":{\"field_of_application\":{\"type\":\"string\",\"enum\":[\"unknow"
    "n\",\"SLS\",\"TLS\",\"KLS\",\"MLS\",\"ULS\",\"ALS\",\"BLS\",\"ILS\"],\"description\":\"Fie"
    "ld of application\"},\"comments\":{\"type\":\"string\",\"description\":\"Comment"
    "s\"},\"project\":{\"type\":\"string\",\"description\":\"Project name\"}},\"type\":\""
    "object\",\"$schema\":\"http://json-schema.org/draft-04/schema#\",\"descripti"
    "on\":\"Details about point cloud files\",\"title\":\"Point Cloud "
    "Information\"}"
    ,
    /* example: */
    "{\"field_of_application\":\"ALS\",\"comments\":\"Line 3\",\"project\":\"Campaign "
    "4\"}"
};

// Estimated position and orientation information
const MetaDataItemInfo RDB_RIEGL_POSE_ESTIMATION = {
    /* name        */ "riegl.pose_estimation",
    /* title       */ "Pose Estimation",
    /* description */ "Estimated position and orientation information",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"required\":[\"orientation\"],\"type\":\"object\",\"$schema\":\"http://json-sch"
    "ema.org/draft-04/schema#\",\"title\":\"Pose "
    "Estimation\",\"description\":\"Estimated position and orientation "
    "information as measured by GNSS, IMU or inclination sensors\",\"properti"
    "es\":{\"orientation\":{\"required\":[\"roll\",\"pitch\",\"yaw\",\"roll_accuracy\",\""
    "pitch_accuracy\",\"yaw_accuracy\"],\"type\":\"object\",\"description\":\"Orienta"
    "tion values and orientation accuracies, measured with IMU or "
    "inclination sensors.\",\"properties\":{\"yaw\":{\"type\":\"number\",\"maximum\":3"
    "60,\"description\":\"Yaw angle about scanner Z-axis [deg]\",\"minimum\":-360"
    "},\"yaw_accuracy\":{\"type\":\"number\",\"description\":\"Yaw angle accuracy [d"
    "eg]\",\"exclusiveMinimum\":true,\"minimum\":0},\"pitch\":{\"type\":\"number\",\"ma"
    "ximum\":360,\"description\":\"Pitch angle about scanner Y-axis [deg]\",\"min"
    "imum\":-360},\"roll_accuracy\":{\"type\":\"number\",\"description\":\"Roll angle"
    " accuracy [deg]\",\"exclusiveMinimum\":true,\"minimum\":0},\"roll\":{\"type\":\""
    "number\",\"maximum\":360,\"description\":\"Roll angle about scanner X-axis ["
    "deg]\",\"minimum\":-360},\"pitch_accuracy\":{\"type\":\"number\",\"description\":"
    "\"Pitch angle accuracy [deg]\",\"exclusiveMinimum\":true,\"minimum\":0}}},\"p"
    "osition\":{\"required\":[\"coordinate_1\",\"coordinate_2\",\"coordinate_3\",\"ho"
    "rizontal_accuracy\",\"vertical_accuracy\",\"crs\"],\"type\":\"object\",\"descrip"
    "tion\":\"Position coordinates and position accuracy values as measured "
    "by GNSS in the specified Coordinate Reference System (CRS)\",\"propertie"
    "s\":{\"coordinate_1\":{\"type\":\"number\",\"description\":\"Coordinate 1 as "
    "defined by axis 1 of the specified CRS (e.g., X, Latitude)\"},\"horizont"
    "al_accuracy\":{\"type\":\"number\",\"description\":\"Horizontal accuracy [m]\","
    "\"exclusiveMinimum\":true,\"minimum\":0},\"crs\":{\"required\":[\"epsg\"],\"type\""
    ":\"object\",\"description\":\"Global Coordinate Reference "
    "System\",\"properties\":{\"epsg\":{\"type\":\"integer\",\"description\":\"EPSG "
    "code\",\"minimum\":0},\"wkt\":{\"type\":\"string\",\"description\":\"\\\"Well-Known "
    "Text\\\" string, OGC WKT dialect (see http://www.opengeospatial.org/stan"
    "dards/wkt-crs)\"}}},\"coordinate_2\":{\"type\":\"number\",\"description\":\"Coor"
    "dinate 2 as defined by axis 2 of the specified CRS (e.g., Y, "
    "Longitude)\"},\"coordinate_3\":{\"type\":\"number\",\"description\":\"Coordinate"
    " 3 as defined by axis 3 of the specified CRS (e.g., Z, Altitude)\"},\"ve"
    "rtical_accuracy\":{\"type\":\"number\",\"description\":\"Vertical accuracy [m]"
    "\",\"exclusiveMinimum\":true,\"minimum\":0}}},\"barometric_height_amsl\":{\"ty"
    "pe\":\"number\",\"description\":\"Altitude determined based on the "
    "atmospheric pressure according to the standard atmosphere laws "
    "[m].\"}}}"
    ,
    /* example: */
    "{\"orientation\":{\"yaw\":101.87293630292045,\"yaw_accuracy\":1.009433783936"
    "8757,\"pitch\":1.509153024827064,\"roll_accuracy\":0.009433783936875745,\"r"
    "oll\":3.14743073066123,\"pitch_accuracy\":0.009433783936875745},\"position"
    "\":{\"coordinate_1\":48.655799473,\"horizontal_accuracy\":0.810699999332428"
    ",\"crs\":{\"epsg\":4979,\"wkt\":\"GEOGCS[\\\"WGS84 / Geographic\\\",DATUM[\\\"WGS84"
    "\\\",SPHEROID[\\\"WGS84\\\",6378137.000,298.257223563,AUTHORITY[\\\"EPSG\\\",\\\"7"
    "030\\\"]],AUTHORITY[\\\"EPSG\\\",\\\"6326\\\"]],PRIMEM[\\\"Greenwich\\\",0.000000000"
    "0000000,AUTHORITY[\\\"EPSG\\\",\\\"8901\\\"]],UNIT[\\\"Degree\\\",0.01745329251994"
    "329547,AUTHORITY[\\\"EPSG\\\",\\\"9102\\\"]],AXIS[\\\"Latitude\\\",NORTH],AXIS[\\\"L"
    "ongitude\\\",EAST],AUTHORITY[\\\"EPSG\\\",\\\"4979\\\"]]\"},\"coordinate_2\":15.645"
    "033406,\"coordinate_3\":362.7124938964844,\"vertical_accuracy\":1.33149993"
    "41964722},\"barometric_height_amsl\":386.7457796227932}"
};

// Details on position and orientation sensors
const MetaDataItemInfo RDB_RIEGL_POSE_SENSORS = {
    /* name        */ "riegl.pose_sensors",
    /* title       */ "Pose Sensors",
    /* description */ "Details on position and orientation sensors",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"required\":[\"gyroscope\",\"accelerometer\",\"magnetic_field_sensor\"],\"typ"
    "e\":\"object\",\"$schema\":\"http://json-schema.org/draft-04/schema#\",\"title"
    "\":\"Pose Sensors\",\"description\":\"Details on position and orientation se"
    "nsors\",\"definitions\":{\"vector\":{\"minItems\":3,\"type\":\"array\",\"items\":{\""
    "type\":\"number\",\"description\":\"Index 0=X, 1=Y, 2=Z component\"},\"maxItem"
    "s\":3}},\"properties\":{\"magnetic_field_sensor\":{\"required\":[\"unit\",\"x_ax"
    "is\",\"y_axis\",\"z_axis\",\"offset\",\"fixed\",\"relative_nonlinearity\"],\"type\""
    ":\"object\",\"description\":\"Magnetic Field Sensor details\",\"properties\":{"
    "\"relative_nonlinearity\":{\"description\":\"Relative nonlinearity, dimensi"
    "onless\",\"$ref\":\"#/definitions/vector\"},\"unit\":{\"type\":\"number\",\"descri"
    "ption\":\"Unit of raw data and calibration values, 1 LSB in nT\",\"exclusi"
    "veMinimum\":true,\"minimum\":0},\"fixed\":{\"description\":\"Distortion of "
    "magnetic field caused by non-rotating scanner part\",\"$ref\":\"#/definiti"
    "ons/vector\"},\"x_axis\":{\"description\":\"Sensitive X axis of sensor at "
    "frame angle = "
    "0\",\"$ref\":\"#/definitions/vector\"},\"y_axis\":{\"description\":\"Sensitive Y"
    " axis of sensor at frame angle = "
    "0\",\"$ref\":\"#/definitions/vector\"},\"offset\":{\"description\":\"Value to be"
    " subtracted from raw measurement values\",\"$ref\":\"#/definitions/vector\""
    "},\"z_axis\":{\"description\":\"Sensitive Z axis of sensor at frame angle ="
    " 0\",\"$ref\":\"#/definitions/vector\"}}},\"gyroscope\":{\"required\":[\"unit\",\""
    "x_axis\",\"y_axis\",\"z_axis\",\"offset\",\"origin\",\"relative_nonlinearity\"],\""
    "type\":\"object\",\"description\":\"Gyroscope details\",\"properties\":{\"relati"
    "ve_nonlinearity\":{\"description\":\"Relative nonlinearity, dimensionless\""
    ",\"$ref\":\"#/definitions/vector\"},\"unit\":{\"type\":\"number\",\"description\":"
    "\"Unit of raw data and calibration values, 1 LSB in rad/s\",\"exclusiveMi"
    "nimum\":true,\"minimum\":0},\"x_axis\":{\"description\":\"Sensitive X axis of "
    "sensor at frame angle = "
    "0\",\"$ref\":\"#/definitions/vector\"},\"origin\":{\"description\":\"Sensor "
    "origin in SOCS [m] at frame angle = "
    "0\",\"$ref\":\"#/definitions/vector\"},\"y_axis\":{\"description\":\"Sensitive Y"
    " axis of sensor at frame angle = "
    "0\",\"$ref\":\"#/definitions/vector\"},\"offset\":{\"description\":\"Value to be"
    " subtracted from raw measurement values\",\"$ref\":\"#/definitions/vector\""
    "},\"z_axis\":{\"description\":\"Sensitive Z axis of sensor at frame angle ="
    " 0\",\"$ref\":\"#/definitions/vector\"}}},\"accelerometer\":{\"required\":[\"uni"
    "t\",\"x_axis\",\"y_axis\",\"z_axis\",\"offset\",\"origin\",\"relative_nonlinearity"
    "\"],\"type\":\"object\",\"description\":\"Accelerometer details\",\"properties\":"
    "{\"relative_nonlinearity\":{\"description\":\"Relative nonlinearity, dimens"
    "ionless\",\"$ref\":\"#/definitions/vector\"},\"unit\":{\"type\":\"number\",\"descr"
    "iption\":\"Unit of raw data and calibration values, 1 LSB in 9.81 m/s\\u0"
    "0b2\",\"exclusiveMinimum\":true,\"minimum\":0},\"x_axis\":{\"description\":\"Sen"
    "sitive X axis of sensor at frame angle = "
    "0\",\"$ref\":\"#/definitions/vector\"},\"origin\":{\"description\":\"Sensor "
    "origin in SOCS [m] at frame angle = "
    "0\",\"$ref\":\"#/definitions/vector\"},\"y_axis\":{\"description\":\"Sensitive Y"
    " axis of sensor at frame angle = "
    "0\",\"$ref\":\"#/definitions/vector\"},\"offset\":{\"description\":\"Value to be"
    " subtracted from raw measurement values\",\"$ref\":\"#/definitions/vector\""
    "},\"z_axis\":{\"description\":\"Sensitive Z axis of sensor at frame angle ="
    " 0\",\"$ref\":\"#/definitions/vector\"}}}}}"
    ,
    /* example: */
    "{\"magnetic_field_sensor\":{\"relative_nonlinearity\":[0.0,0.0,0.0],\"unit\""
    ":91.74311828613281,\"fixed\":[-1576.010498046875,1596.081787109375,0.0],"
    "\"x_axis\":[-0.011162743903696537,-2.315962774446234e-05,0.0001681884459"
    "6855342],\"y_axis\":[0.00027888521435670555,-0.011427424848079681,-5.204"
    "829722060822e-05],\"offset\":[-23812.052734375,5606.57666015625,2493.281"
    "25],\"z_axis\":[0.00041987866279669106,7.876977906562388e-05,0.011407104"
    "320824146]},\"gyroscope\":{\"relative_nonlinearity\":[2.888176311444113e-0"
    "7,1.06274164579645e-07,-1.7186295080634935e-39],\"unit\":0.0001454441080"
    "5683583,\"x_axis\":[-121.195556640625,0.8219714164733887,0.2313031703233"
    "719],\"origin\":[0.026900000870227814,-0.03999999910593033,-0.0895000025"
    "6299973],\"y_axis\":[-0.440765917301178,-0.7897399663925171,119.58947753"
    "90625],\"offset\":[-50.92609786987305,146.15643310546875,62.432727813720"
    "7],\"z_axis\":[0.555869996547699,119.22135162353516,0.467585027217865]},"
    "\"accelerometer\":{\"relative_nonlinearity\":[0.0,0.0,0.0],\"unit\":6.666666"
    "740784422e-05,\"x_axis\":[-15008.123046875,56.956390380859375,-60.517566"
    "6809082],\"origin\":[0.026900000870227814,-0.03999999910593033,-0.089500"
    "00256299973],\"y_axis\":[-7.027288913726807,-44.12333679199219,14952.370"
    "1171875],\"offset\":[-733.3636474609375,58.969032287597656,1060.25500488"
    "28125],\"z_axis\":[1.639882206916809,15166.744140625,-116.99742889404297"
    "]}}"
};

// Laser pulse position modulation used for MTA resolution
const MetaDataItemInfo RDB_RIEGL_PULSE_POSITION_MODULATION = {
    /* name        */ "riegl.pulse_position_modulation",
    /* title       */ "Pulse Position Modulation",
    /* description */ "Laser pulse position modulation used for MTA resolution",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"required\":[\"length\",\"num_mod_ampl\",\"pulse_interval\"],\"type\":\"object\""
    ",\"$schema\":\"http://json-schema.org/draft-04/schema#\",\"title\":\"Pulse "
    "Position Modulation\",\"description\":\"Laser pulse position modulation "
    "used for MTA resolution\",\"properties\":{\"length\":{\"type\":\"integer\",\"max"
    "imum\":255,\"description\":\"Length of code\",\"minimum\":0},\"phase_step\":{\"t"
    "ype\":\"integer\",\"maximum\":255,\"description\":\"Step width in phase of "
    "modulation code from line to line\",\"minimum\":0},\"pulse_interval\":{\"typ"
    "e\":\"array\",\"description\":\"Explicit table of the pulse position "
    "modulation used for MTA resolution. Table gives times between "
    "successive laser pulses in seconds.\",\"items\":{\"type\":\"number\",\"minimum"
    "\":0}},\"code_phase_mode\":{\"type\":\"integer\",\"maximum\":255,\"description\":"
    "\"0: no synchronization, 1: toggle between 2 phases, 2: increment with "
    "phase_increment\",\"minimum\":0},\"num_mod_ampl\":{\"type\":\"integer\",\"maximu"
    "m\":255,\"description\":\"Number of different modulation amplitudes (2: "
    "binary modulation)\",\"minimum\":0}}}"
    ,
    /* example: */
    "{\"length\":31,\"phase_step\":5,\"pulse_interval\":[2.759375e-06,2.759375e-0"
    "6,2.759375e-06,2.759375e-06,2.821875e-06,2.759375e-06,2.759375e-06,2.8"
    "21875e-06,2.759375e-06,2.821875e-06,2.821875e-06,2.759375e-06,2.759375"
    "e-06,2.821875e-06,2.821875e-06,2.821875e-06,2.821875e-06,2.821875e-06,"
    "2.759375e-06,2.759375e-06,2.759375e-06,2.821875e-06,2.821875e-06,2.759"
    "375e-06,2.821875e-06,2.821875e-06,2.821875e-06,2.759375e-06,2.821875e-"
    "06,2.759375e-06,2.821875e-06],\"code_phase_mode\":2,\"num_mod_ampl\":2}"
};

// Statistics about target distance wrt. SOCS origin
const MetaDataItemInfo RDB_RIEGL_RANGE_STATISTICS = {
    /* name        */ "riegl.range_statistics",
    /* title       */ "Range Statistics",
    /* description */ "Statistics about target distance wrt. SOCS origin",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"required\":[\"minimum\",\"average\",\"maximum\",\"std_dev\"],\"type\":\"object\","
    "\"$schema\":\"http://json-schema.org/draft-04/schema#\",\"title\":\"Range "
    "Statistics\",\"description\":\"Statistics about target distance wrt. SOCS "
    "origin\",\"properties\":{\"std_dev\":{\"type\":\"number\",\"description\":\"Standa"
    "rd deviation\"},\"maximum\":{\"type\":\"number\",\"description\":\"Maximum "
    "value\"},\"average\":{\"type\":\"number\",\"description\":\"Average "
    "value\"},\"minimum\":{\"type\":\"number\",\"description\":\"Minimum value\"}}}"
    ,
    /* example: */
    "{\"std_dev\":24.349,\"maximum\":574.35,\"average\":15.49738,\"minimum\":0.919}"
};

// Receiver Internals
const MetaDataItemInfo RDB_RIEGL_RECEIVER_INTERNALS = {
    /* name        */ "riegl.receiver_internals",
    /* title       */ "Receiver Internals",
    /* description */ "Receiver Internals",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"type\":\"object\",\"$schema\":\"http://json-schema.org/draft-04/schema#\",\""
    "title\":\"Receiver Internals\",\"description\":\"Receiver Internals\",\"defini"
    "tions\":{\"fp_table\":{\"type\":\"object\",\"required\":[\"ch\",\"tc\",\"nx\",\"ny\",\"t"
    "v\"],\"desription\":\"scanner internal data\",\"properties\":{\"ch\":{\"type\":\"i"
    "nteger\",\"min\":0,\"description\":\"channel number\",\"max\":255},\"ny\":{\"type\""
    ":\"integer\",\"min\":1,\"description\":\"number of y entries\",\"max\":2048},\"nx"
    "\":{\"type\":\"integer\",\"min\":1,\"description\":\"number of x entries\",\"max\":"
    "2048},\"tc\":{\"type\":\"integer\",\"min\":0,\"description\":\"table data type co"
    "de\",\"max\":255},\"tv\":{\"minItems\":1,\"type\":\"array\",\"items\":{\"oneOf\":[{\"$"
    "ref\":\"#/definitions/fp_table_row\"},{\"type\":\"number\"}]},\"maxItems\":2048"
    "}}},\"fp_table_row\":{\"minItems\":1,\"type\":\"array\",\"items\":{\"type\":\"numbe"
    "r\"},\"maxItems\":2048},\"fp\":{\"required\":[\"s\",\"w\"],\"type\":\"object\",\"descr"
    "iption\":\"Fingerprint values\",\"properties\":{\"s\":{\"minItems\":1,\"type\":\"a"
    "rray\",\"items\":{\"minItems\":1,\"type\":\"array\",\"items\":{\"type\":\"number\"},\""
    "maxItems\":4096},\"maxItems\":256},\"w\":{\"minItems\":1,\"type\":\"array\",\"item"
    "s\":{\"minItems\":5,\"type\":\"array\",\"items\":{\"type\":\"number\"},\"maxItems\":5"
    "},\"maxItems\":256}}}},\"properties\":{\"si\":{\"type\":\"number\",\"maximum\":255"
    ",\"description\":\"Start index (hw_start)\",\"minimum\":0},\"ex\":{\"type\":\"obj"
    "ect\",\"description\":\"DEPRECATED, use 'riegl.exponential_decomposition' "
    "instead\"},\"nt\":{\"type\":\"integer\",\"maximum\":255,\"description\":\"Number "
    "of traces\",\"minimum\":0},\"t\":{\"type\":\"object\",\"patternProperties\":{\"^[0"
    "-9]+$\":{\"description\":\"one field per channel, field name is channel in"
    "dex\",\"$ref\":\"#/definitions/fp\"}},\"additionalProperties\":false},\"mw\":{\""
    "type\":\"number\",\"maximum\":4095,\"description\":\"Maximum weight\",\"exclusiv"
    "eMinimum\":true,\"minimum\":0},\"ns\":{\"type\":\"integer\",\"maximum\":4095,\"des"
    "cription\":\"Number of samples\",\"minimum\":0},\"tbl\":{\"minItems\":1,\"type\":"
    "\"array\",\"items\":{\"$ref\":\"#/definitions/fp_table\"},\"description\":\"vario"
    "us internal data\"},\"sr\":{\"type\":\"number\",\"description\":\"Sample rate [H"
    "z]\",\"exclusiveMinimum\":true,\"minimum\":0},\"a\":{\"minItems\":1,\"type\":\"arr"
    "ay\",\"items\":{\"type\":\"number\"},\"description\":\"Amplitude "
    "[dB]\",\"maxItems\":256}}}"
    ,
    /* example: */
    "{\"si\":48,\"nt\":128,\"t\":{\"0\":{\"s\":[[1.23,4.56],[7.89,0.12]],\"w\":[[78,86,"
    "126,134,31],[78,86,126,134,31]]},\"1\":{\"s\":[[1.23,4.56],[7.89,0.12]],\"w"
    "\":[[78,86,126,134,31],[78,86,126,134,31]]}},\"mw\":31,\"ns\":400,\"tbl\":[{\""
    "ch\":0,\"ny\":2,\"nx\":5,\"tc\":1,\"tv\":[[1,2,3,4,5],[1.1,2.2,3.3,4.4,5.5]]}],"
    "\"sr\":7959997000.0,\"a\":[-1.55]}"
};

// Lookup table for reflectance calculation based on amplitude and range
const MetaDataItemInfo RDB_RIEGL_REFLECTANCE_CALCULATION = {
    /* name        */ "riegl.reflectance_calculation",
    /* title       */ "Reflectance Calculation Table",
    /* description */ "Lookup table for reflectance calculation based on amplitude and range",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"required\":[\"delta\",\"content\"],\"type\":\"object\",\"$schema\":\"http://json"
    "-schema.org/draft-04/schema#\",\"title\":\"Reflectance Calculation "
    "Table\",\"description\":\"Lookup table for reflectance calculation based "
    "on amplitude and range\",\"properties\":{\"content\":{\"minItems\":1,\"type\":\""
    "array\",\"description\":\"Correction value [dB] to be added to the amplitu"
    "de\",\"items\":{\"type\":\"number\"},\"maxItems\":2000},\"delta\":{\"type\":\"number"
    "\",\"description\":\"Delta between table entries [m], first entry is at "
    "range = 0 m\"}}}"
    ,
    /* example: */
    "{\"content\":[-33.01],\"delta\":0.150918}"
};

// Range-dependent and scan-angle-dependent correction of reflectance reading
const MetaDataItemInfo RDB_RIEGL_REFLECTANCE_CORRECTION = {
    /* name        */ "riegl.reflectance_correction",
    /* title       */ "Near-Range Reflectance Correction",
    /* description */ "Range-dependent and scan-angle-dependent correction of reflectance reading",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"required\":[\"ranges_m\",\"line_angles_deg\",\"reflectance_correction_db\"]"
    ",\"type\":\"object\",\"$schema\":\"http://json-schema.org/draft-04/schema#\",\""
    "title\":\"Near-range reflectance "
    "correction\",\"description\":\"Range-dependent and scan-angle-dependent "
    "correction of reflectance reading\",\"properties\":{\"line_angles_deg\":{\"t"
    "ype\":\"array\",\"description\":\"Angle [deg]\",\"items\":{\"type\":\"number\"}},\"r"
    "eflectance_correction_db\":{\"type\":\"array\",\"description\":\"Near range "
    "reflectance correction in dB as a function of range over "
    "angle\",\"items\":{\"type\":\"array\",\"description\":\"rows (each array "
    "corresponds to a "
    "range)\",\"items\":{\"type\":\"number\",\"description\":\"columns (each value "
    "corresponds to an "
    "angle)\"}}},\"ranges_m\":{\"type\":\"array\",\"description\":\"Range "
    "[m]\",\"items\":{\"type\":\"number\"}}}}"
    ,
    /* example: */
    "{\"line_angles_deg\":[0.0,0.5,1.0,1.5,1.0,2.0,2.5,3.0,3.5,4.0],\"reflecta"
    "nce_correction_db\":[[0.8,0.7,0.6,0.5,0.4,0.3,0.2,0.1,0.05,0.01],[0.8,0"
    ".7,0.6,0.5,0.4,0.3,0.2,0.1,0.05,0.01],[0.8,0.7,0.6,0.5,0.4,0.3,0.2,0.1"
    ",0.05,0.01],[0.8,0.7,0.6,0.5,0.4,0.3,0.2,0.1,0.05,0.01]],\"ranges_m\":[0"
    ".0,1.0,2.0,3.0]}"
};

// Scan pattern description
const MetaDataItemInfo RDB_RIEGL_SCAN_PATTERN = {
    /* name        */ "riegl.scan_pattern",
    /* title       */ "Scan Pattern",
    /* description */ "Scan pattern description",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"properties\":{\"rectangular\":{\"required\":[\"phi_start\",\"phi_stop\",\"phi_"
    "increment\",\"theta_start\",\"theta_stop\",\"theta_increment\"],\"type\":\"objec"
    "t\",\"description\":\"Rectangular Field Of View Scan Pattern\",\"properties\""
    ":{\"phi_start\":{\"type\":\"number\",\"maximum\":360.0,\"description\":\"Start "
    "phi angle in SOCS [deg]\",\"minimum\":0.0},\"program\":{\"$ref\":\"#/definitio"
    "ns/program\"},\"theta_stop\":{\"type\":\"number\",\"maximum\":180.0,\"descriptio"
    "n\":\"Stop theta angle in SOCS [deg]\",\"minimum\":0.0},\"theta_start\":{\"typ"
    "e\":\"number\",\"maximum\":180.0,\"description\":\"Start theta angle in SOCS ["
    "deg]\",\"minimum\":0.0},\"phi_stop\":{\"type\":\"number\",\"maximum\":720.0,\"desc"
    "ription\":\"Stop phi angle in SOCS [deg]\",\"minimum\":0.0},\"phi_increment\""
    ":{\"type\":\"number\",\"maximum\":90.0,\"description\":\"Increment of phi angle"
    " in SOCS [deg]\",\"minimum\":0.0},\"theta_increment\":{\"type\":\"number\",\"max"
    "imum\":90.0,\"description\":\"Increment of theta angle in SOCS [deg]\",\"min"
    "imum\":0.0}}},\"segments\":{\"required\":[\"list\"],\"type\":\"object\",\"descript"
    "ion\":\"Segmented Line Scan Pattern\",\"properties\":{\"list\":{\"type\":\"array"
    "\",\"items\":{\"required\":[\"start\",\"stop\",\"increment\"],\"type\":\"object\",\"de"
    "scription\":\"Line Scan Segment\",\"properties\":{\"start\":{\"type\":\"number\","
    "\"maximum\":360.0,\"description\":\"Start angle in SOCS [deg]\",\"minimum\":0."
    "0},\"stop\":{\"type\":\"number\",\"maximum\":720.0,\"description\":\"Stop angle "
    "in SOCS [deg]\",\"minimum\":0.0},\"increment\":{\"type\":\"number\",\"maximum\":9"
    "0.0,\"description\":\"Increment of angle in SOCS [deg]\",\"minimum\":0.0}}}}"
    ",\"program\":{\"$ref\":\"#/definitions/program\"}}},\"line\":{\"required\":[\"sta"
    "rt\",\"stop\",\"increment\"],\"type\":\"object\",\"description\":\"Line Scan Patte"
    "rn\",\"properties\":{\"program\":{\"$ref\":\"#/definitions/program\"},\"start\":{"
    "\"type\":\"number\",\"maximum\":360.0,\"description\":\"Start angle in SOCS [de"
    "g]\",\"minimum\":0.0},\"stop\":{\"type\":\"number\",\"maximum\":720.0,\"descriptio"
    "n\":\"Stop angle in SOCS [deg]\",\"minimum\":0.0},\"increment\":{\"type\":\"numb"
    "er\",\"maximum\":90.0,\"description\":\"Increment of angle in SOCS [deg]\",\"m"
    "inimum\":0.0}}}},\"definitions\":{\"program\":{\"required\":[\"name\"],\"type\":\""
    "object\",\"description\":\"Measurement program\",\"properties\":{\"laser_prr\":"
    "{\"type\":\"number\",\"description\":\"Laser Pulse Repetition Rate [Hz]\",\"exc"
    "lusiveMinimum\":false,\"minimum\":0},\"name\":{\"type\":\"string\",\"description"
    "\":\"Name of measurement program\"}}}},\"$schema\":\"http://json-schema.org/"
    "draft-04/schema#\",\"description\":\"Scan pattern "
    "description\",\"title\":\"Scan Pattern\"}"
    ,
    /* example: */
    "{\"rectangular\":{\"phi_start\":45.0,\"program\":{\"laser_prr\":100000.0,\"name"
    "\":\"High Speed\"},\"theta_stop\":130.0,\"theta_start\":30.0,\"phi_stop\":270.0"
    ",\"phi_increment\":0.04,\"theta_increment\":0.04}}"
};

// Details about laser shot files
const MetaDataItemInfo RDB_RIEGL_SHOT_INFO = {
    /* name        */ "riegl.shot_info",
    /* title       */ "Shot Information",
    /* description */ "Details about laser shot files",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"properties\":{\"shot_file\":{\"type\":\"object\",\"required\":[\"file_extensio"
    "n\"],\"properties\":{\"file_uuid\":{\"type\":\"string\",\"description\":\"File's "
    "Universally Unique Identifier (RFC "
    "4122)\"},\"file_extension\":{\"type\":\"string\",\"description\":\"Shot file "
    "extension, without the leading dot\"}}}},\"type\":\"object\",\"$schema\":\"htt"
    "p://json-schema.org/draft-04/schema#\",\"description\":\"Details about "
    "laser shot files\",\"title\":\"Shot Information\"}"
    ,
    /* example: */
    "{\"shot_file\":{\"file_uuid\":\"26a00815-67c0-4bff-8fe8-c577378fe663\",\"file"
    "_extension\":\"sodx\"}}"
};

// Point filters applied in addition to the application-defined filters
const MetaDataItemInfo RDB_RIEGL_STORED_FILTERS = {
    /* name        */ "riegl.stored_filters",
    /* title       */ "Stored Filters",
    /* description */ "Point filters applied in addition to the application-defined filters",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"required\":[\"activated\",\"filters\"],\"type\":\"object\",\"$schema\":\"http://"
    "json-schema.org/draft-04/schema#\",\"title\":\"Stored "
    "filters\",\"description\":\"Point filters applied in addition to the "
    "application-defined filters\",\"properties\":{\"activated\":{\"type\":\"boolea"
    "n\",\"description\":\"Apply ('true') or ignore ('false') all "
    "filters\"},\"filters\":{\"type\":\"array\",\"description\":\"List of point filte"
    "rs\",\"items\":{\"required\":[\"activated\",\"title\",\"description\",\"filter\"],\""
    "type\":\"object\",\"description\":\"Point filter definition\",\"properties\":{\""
    "activated\":{\"type\":\"boolean\",\"description\":\"Apply ('true') or ignore "
    "('false') this "
    "filter\"},\"description\":{\"type\":\"string\",\"description\":\"A brief "
    "description of the filter (e.g. for display in a "
    "tooltip)\"},\"filter\":{\"type\":\"string\",\"description\":\"The RDB filter "
    "string to apply (e.g. when reading points or index), details see "
    "documentation of function "
    "select()\"},\"title\":{\"type\":\"string\",\"description\":\"A short filter "
    "title (e.g. for display in a selection list)\"}}}}}}"
    ,
    /* example: */
    "{\"activated\":true,\"filters\":[{\"activated\":false,\"description\":\"Ignore "
    "points with uncertain MTA zone "
    "assignment\",\"filter\":\"riegl.mta_uncertain_point == 0\",\"title\":\"Ignore "
    "uncertain points from MTA "
    "resolution\"},{\"activated\":false,\"description\":\"Ignore points with an "
    "echo signal amplitude below the dynamic "
    "trigger\",\"filter\":\"riegl.dyntrig_uncertain_point == 0\",\"title\":\"Ignore"
    " points below dynamic trigger "
    "level\"},{\"activated\":false,\"description\":\"Ignore points that are "
    "outside the outer AABB in "
    "BOCS\",\"filter\":\"riegl.point_outside_aabb_bocs == 0\",\"title\":\"Ignore "
    "points outside outer AABB\"},{\"activated\":false,\"description\":\"Ignore "
    "points that are inside the inner AABB in "
    "BOCS\",\"filter\":\"riegl.point_inside_aabb_bocs == 0\",\"title\":\"Ignore "
    "points inside inner AABB\"}]}"
};

// Details of major system components like lidar sensors, cameras and navigation devices
const MetaDataItemInfo RDB_RIEGL_SYSTEM_DESCRIPTION = {
    /* name        */ "riegl.system_description",
    /* title       */ "System Description",
    /* description */ "Details of major system components like lidar sensors, cameras and navigation devices",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"required\":[\"$class\",\"version\",\"author\",\"timestamp\",\"system\"],\"type\":"
    "\"object\",\"$schema\":\"http://json-schema.org/draft-04/schema#\",\"title\":\""
    "System Description\",\"description\":\"Details of major system components "
    "like lidar sensors, cameras and navigation "
    "devices\",\"properties\":{\"author\":{\"type\":\"string\",\"description\":\"Name "
    "of the author that created the "
    "document\"},\"timestamp\":{\"type\":\"string\",\"description\":\"Date and time "
    "of creation (e.g. 2019-06-19T13:32:10+02:00)\"},\"$class\":{\"type\":\"strin"
    "g\",\"enum\":[\"Document\"],\"description\":\"Object class "
    "name\"},\"system\":{\"type\":\"object\",\"description\":\"The actual system "
    "description, details see documentation of RIEGL System "
    "Description\"},\"version\":{\"type\":\"string\",\"description\":\"Document "
    "format version\"}}}"
    ,
    /* example: */
    "{\"author\":\"RIEGL LMS GmbH, Austria\",\"timestamp\":\"2022-09-30T11:56:26+0"
    "0:00\",\"$class\":\"Document\",\"system\":{\"details\":\"see documentation of "
    "RIEGL System Description\"},\"version\":\"1.2.1\"}"
};

// Conversion of background radiation raw values to temperatures in °C
const MetaDataItemInfo RDB_RIEGL_TEMPERATURE_CALCULATION = {
    /* name        */ "riegl.temperature_calculation",
    /* title       */ "Temperature Calculation Table",
    /* description */ "Conversion of background radiation raw values to temperatures in °C",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"type\":\"object\",\"$schema\":\"http://json-schema.org/draft-04/schema#\",\""
    "title\":\"Temperature Calculation Table\",\"description\":\"Conversion of "
    "background radiation raw values to temperatures in \\u00b0C\",\"definitio"
    "ns\":{\"conversion_table\":{\"type\":\"object\",\"required\":[\"value\",\"temperat"
    "ure\"],\"properties\":{\"temperature\":{\"type\":\"array\",\"description\":\"Tempe"
    "rature [\\u00b0C]\",\"items\":{\"type\":\"number\"}},\"value\":{\"type\":\"array\",\""
    "description\":\"LSB [1]\",\"items\":{\"type\":\"number\"}}}}},\"properties\":{\"In"
    "GaAs\":{\"description\":\"Conversion table for InGaAs channel\",\"$ref\":\"#/d"
    "efinitions/conversion_table\"},\"InGaAs_Si_Difference\":{\"description\":\"C"
    "onversion table for InGaAs - Si difference\",\"$ref\":\"#/definitions/conv"
    "ersion_table\"},\"Si\":{\"description\":\"Conversion table for Si "
    "channel\",\"$ref\":\"#/definitions/conversion_table\"}}}"
    ,
    /* example: */
    "{\"InGaAs\":{\"temperature\":[307.22196722535614,309.1153478247277,311.118"
    "8086915047,313.10025350480055,315.2137946389828,317.2172555057597,319."
    "2207163725366,321.2021611858325,323.3157023200148],\"value\":[0.0,64.000"
    "97659230323,128.0019531846065,192.0029297769097,256.0039063692129,320."
    "00488296151616,384.0058595538194,448.0068361461226,512.0078127384259]}"
    ",\"InGaAs_Si_Difference\":{\"temperature\":[1749.977111117893,1749.9771111"
    "17893,1749.977111117893,1749.977111117893,1749.977111117893,1749.97711"
    "1117893,1744.7813348796044,1681.9971312601092,1622.3944822534868],\"val"
    "ue\":[1000.0,1100.090029602954,1200.04425183874,1300.1342814416948,1400"
    ".0885036774805,1500.0427259132668,1600.1327555162209,1700.086977752006"
    "5,1800.0411999877924]},\"Si\":{\"temperature\":[546.300048828125,548.81640"
    "51212026,551.3143938500972,554.0144257850053,556.604252334815,559.2124"
    "464488079,561.8022729986177,564.4104671126105,567.0002936624203],\"valu"
    "e\":[0.0,64.00097659230323,128.0019531846065,192.0029297769097,256.0039"
    "063692129,320.00488296151616,384.0058595538194,448.0068361461226,512.0"
    "078127384259]}}"
};

// Base of timestamps (epoch)
const MetaDataItemInfo RDB_RIEGL_TIME_BASE = {
    /* name        */ "riegl.time_base",
    /* title       */ "Time Base",
    /* description */ "Base of timestamps (epoch)",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"required\":[\"epoch\",\"source\"],\"type\":\"object\",\"$schema\":\"http://json-"
    "schema.org/draft-04/schema#\",\"title\":\"Time Base\",\"description\":\"Base "
    "of timestamps "
    "(epoch)\",\"properties\":{\"epoch\":{\"type\":\"string\",\"description\":\"Date "
    "and time of timestamp '0' as proposed by RFC 3339 (e.g. 2015-10-27T00:"
    "00:00+01:00).\"},\"system\":{\"type\":\"string\",\"enum\":[\"unknown\",\"UTC\",\"GPS"
    "\"],\"description\":\"Time system (time standard)\"},\"source\":{\"type\":\"stri"
    "ng\",\"enum\":[\"unknown\",\"RTC\",\"GNSS\"],\"description\":\"Timestamp "
    "source\"}}}"
    ,
    /* example: */
    "{\"epoch\":\"2015-10-27T00:00:00+00:00\",\"system\":\"UTC\",\"source\":\"GNSS\"}"
};

// Details about position+orientation files
const MetaDataItemInfo RDB_RIEGL_TRAJECTORY_INFO = {
    /* name        */ "riegl.trajectory_info",
    /* title       */ "Trajectory Information",
    /* description */ "Details about position+orientation files",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"required\":[\"time_interval\",\"navigation_frame\"],\"type\":\"object\",\"$sch"
    "ema\":\"http://json-schema.org/draft-04/schema#\",\"title\":\"Trajectory "
    "Information\",\"description\":\"Details about position+orientation files\","
    "\"properties\":{\"settings\":{\"type\":\"string\",\"description\":\"Settings used"
    " to calculate the trajectory (descriptive "
    "text)\"},\"company\":{\"type\":\"string\",\"description\":\"Company "
    "name\"},\"software\":{\"type\":\"string\",\"description\":\"Software that "
    "calculated the trajectory (this may be the same or different software "
    "than the one that created the "
    "file)\"},\"location\":{\"type\":\"string\",\"description\":\"Project location, "
    "e.g. city/state/country\"},\"field_of_application\":{\"type\":\"string\",\"enu"
    "m\":[\"unknown\",\"SLS\",\"TLS\",\"KLS\",\"MLS\",\"ULS\",\"ALS\",\"BLS\",\"ILS\"],\"descri"
    "ption\":\"Field of application\"},\"time_interval\":{\"required\":[\"minimum\","
    "\"average\",\"maximum\",\"std_dev\"],\"type\":\"object\",\"description\":\"Time "
    "interval statistics\",\"properties\":{\"std_dev\":{\"type\":\"number\",\"descrip"
    "tion\":\"Standard deviation of intervals "
    "[s]\"},\"maximum\":{\"type\":\"number\",\"description\":\"Maximum time interval "
    "[s]\"},\"average\":{\"type\":\"number\",\"description\":\"Average time interval "
    "[s]\"},\"minimum\":{\"type\":\"number\",\"description\":\"Minimum time interval "
    "[s]\"}}},\"device\":{\"type\":\"string\",\"description\":\"Navigation device, "
    "e.g. "
    "name/type/serial\"},\"project\":{\"type\":\"string\",\"description\":\"Project n"
    "ame\"},\"navigation_frame\":{\"type\":\"string\",\"enum\":[\"unknown\",\"NED\",\"ENU"
    "\"],\"description\":\"Navigation frame (NED: North-East-Down, ENU: "
    "East-North-Up)\"}}}"
    ,
    /* example: */
    "{\"settings\":\"default\",\"company\":\"RIEGL LMS\",\"software\":\"Navigation "
    "Software XYZ\",\"location\":\"Horn\",\"field_of_application\":\"MLS\",\"time_int"
    "erval\":{\"std_dev\":5.51e-07,\"maximum\":0.005004883,\"average\":0.00500053,"
    "\"minimum\":0.00500032},\"device\":\"IMU Model 12/1, Serial# "
    "12345\",\"project\":\"Campaign 3\",\"navigation_frame\":\"NED\"}"
};

// Trigger-Timestamping-Unit/IP configuration
const MetaDataItemInfo RDB_RIEGL_TTIP_CONFIGURATION = {
    /* name        */ "riegl.ttip_configuration",
    /* title       */ "TTIP Configuration",
    /* description */ "Trigger-Timestamping-Unit/IP configuration",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"required\":[\"ttip_version\",\"num_channel\",\"ext_channel\",\"ext_signal\",\""
    "ext_delay\",\"ext_subdivider\",\"ext_bitmask\",\"dmi_incr\",\"dmi_min_time\",\"d"
    "mi_max_time\",\"dmi_dist_per_tick\",\"channel\",\"output_descr\",\"input_descr"
    "\",\"output_usage\",\"input_usage\",\"out_polarity\",\"out_duration\",\"out_inte"
    "rval\",\"out_delay\",\"out_num_pulses\",\"in_polarity\",\"in_min_duration\",\"in"
    "_max_duration\",\"in_max_delay\"],\"type\":\"object\",\"$schema\":\"http://json-"
    "schema.org/draft-04/schema#\",\"title\":\"TTIP "
    "Configuration\",\"description\":\"Trigger-Timestamping-Unit/IP configurati"
    "on\",\"properties\":{\"output_usage\":{\"type\":\"integer\",\"description\":\"0 .."
    " no output, 1 .. output derived from internal clock, 2 .. output "
    "derived from dmi, 3 .. output derived from external signal, 4 .. "
    "output static low, 5 .. output static high\",\"minimum\":0},\"out_interval"
    "\":{\"type\":\"integer\",\"description\":\"output pulse interval  [0.1 "
    "msec]\",\"minimum\":0},\"channel\":{\"type\":\"integer\",\"description\":\"ID of "
    "input/output channel\",\"minimum\":0},\"dmi_min_time\":{\"type\":\"integer\",\"d"
    "escription\":\"dmi, minimum time interval between trigger events [0.1 ms"
    "ec]\",\"minimum\":0},\"input_descr\":{\"type\":\"string\",\"description\":\"descri"
    "ptive "
    "string\"},\"in_min_duration\":{\"type\":\"integer\",\"description\":\"input "
    "signals with smaller pulse durations are ignored [0.1 msec]\",\"minimum\""
    ":0},\"ext_bitmask\":{\"type\":\"integer\",\"description\":\"defines which of "
    "the internal pulse generators are to be started\",\"minimum\":0},\"ext_cha"
    "nnel\":{\"type\":\"integer\",\"description\":\"ID of channel used as external "
    "trigger input, 32 indicates none\",\"minimum\":0},\"in_max_delay\":{\"type\":"
    "\"integer\",\"description\":\"maximum delay to output pulse before fake "
    "event is generated [0.1 msec], zero indicates that no fake events are "
    "generated\",\"minimum\":0},\"output_descr\":{\"type\":\"string\",\"description\":"
    "\"descriptive "
    "string\"},\"out_num_pulses\":{\"type\":\"integer\",\"description\":\"number of "
    "output pulses to be generated, 0 .. infinite\",\"minimum\":0},\"dmi_max_ti"
    "me\":{\"type\":\"integer\",\"description\":\"dmi, maximum time interval "
    "between trigger events [0.1 "
    "msec]\",\"minimum\":0},\"ext_signal\":{\"type\":\"integer\",\"description\":\"0 .."
    " use input signal, 1 .. use output signal of channel\",\"minimum\":0},\"dm"
    "i_dist_per_tick\":{\"type\":\"number\",\"description\":\"dmi, distance per "
    "tick, just informative "
    "[m]\"},\"out_duration\":{\"type\":\"integer\",\"description\":\"output pulse "
    "duration  [0.1 msec]\",\"minimum\":0},\"ext_delay\":{\"type\":\"integer\",\"desc"
    "ription\":\"external trigger, time delay [0.1 msec]\",\"minimum\":0},\"num_c"
    "hannel\":{\"type\":\"integer\",\"description\":\"number of input/output channe"
    "ls\",\"minimum\":0},\"dmi_incr\":{\"type\":\"integer\",\"description\":\"dmi, "
    "increment in ticks\"},\"out_polarity\":{\"type\":\"integer\",\"description\":\"0"
    " .. positive edge, 1 .. negative "
    "edge\",\"minimum\":0},\"in_polarity\":{\"type\":\"integer\",\"description\":\"0 .."
    " positive edge, 1 .. negative edge\",\"minimum\":0},\"ext_subdivider\":{\"ty"
    "pe\":\"integer\",\"description\":\"reduces the frequency, default 1\",\"minimu"
    "m\":0},\"ttip_version\":{\"type\":\"integer\",\"description\":\"following main.s"
    "ub.ss.sss\",\"minimum\":0},\"out_delay\":{\"type\":\"integer\",\"description\":\"o"
    "utput pulse initial delay after start  [0.1 msec]\",\"minimum\":0},\"in_ma"
    "x_duration\":{\"type\":\"integer\",\"description\":\"stops measurement of "
    "pulse duration of input signal [0.1 "
    "msec]\",\"minimum\":0},\"input_usage\":{\"type\":\"integer\",\"description\":\"0 "
    ".. deactivated, 1 .. just detecting and timestamping\",\"minimum\":0}}}"
    ,
    /* example: */
    "{\"output_usage\":1,\"out_interval\":10000,\"channel\":0,\"dmi_min_time\":0,\"i"
    "nput_descr\":\"Port 1 - Exposure\",\"in_min_duration\":0,\"ext_bitmask\":0,\"e"
    "xt_channel\":32,\"in_max_delay\":9990,\"output_descr\":\"Port 1 - Trigger\",\""
    "out_num_pulses\":1,\"dmi_max_time\":0,\"ext_signal\":0,\"dmi_dist_per_tick\":"
    "0.0,\"out_duration\":300,\"ext_delay\":0,\"num_channel\":9,\"dmi_incr\":0,\"out"
    "_polarity\":0,\"in_polarity\":0,\"ext_subdivider\":1,\"ttip_version\":1,\"out_"
    "delay\":0,\"in_max_duration\":10000,\"input_usage\":1}"
};

// Details about vertex file
const MetaDataItemInfo RDB_RIEGL_VERTEX_INFO = {
    /* name        */ "riegl.vertex_info",
    /* title       */ "Vertex Information",
    /* description */ "Details about vertex file",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"properties\":{\"vertex_file\":{\"type\":\"object\",\"required\":[\"file_extens"
    "ion\"],\"properties\":{\"file_uuid\":{\"type\":\"string\",\"description\":\"File's"
    " Universally Unique Identifier (RFC "
    "4122)\"},\"file_extension\":{\"type\":\"string\",\"description\":\"Vertex file "
    "extension, without the leading dot\"}}}},\"type\":\"object\",\"$schema\":\"htt"
    "p://json-schema.org/draft-04/schema#\",\"description\":\"Details about "
    "vertex file\",\"title\":\"Vertex Information\"}"
    ,
    /* example: */
    "{\"vertex_file\":{\"file_uuid\":\"51534d95-d71f-4f36-ae1a-3e63a21fd1c7\",\"fi"
    "le_extension\":\"vtx\"}}"
};

// Details about the voxels contained in the file
const MetaDataItemInfo RDB_RIEGL_VOXEL_INFO = {
    /* name        */ "riegl.voxel_info",
    /* title       */ "Voxel Information",
    /* description */ "Details about the voxels contained in the file",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"type\":\"object\",\"$schema\":\"http://json-schema.org/draft-04/schema#\",\""
    "oneOf\":[{\"additionalProperties\":false,\"required\":[\"size\",\"voxel_origin"
    "\",\"voxel_type\"],\"properties\":{\"voxel_type\":{\"$ref\":\"#/definitions/voxe"
    "l_type\"},\"shape_thresholds\":{\"$ref\":\"#/definitions/shape_thresholds\"},"
    "\"voxel_origin\":{\"$ref\":\"#/definitions/voxel_origin_enum\"},\"size\":{\"des"
    "cription\":\"Size of voxels in file coordinate system.\",\"oneOf\":[{\"$ref\""
    ":\"#/definitions/voxel_size\"},{\"$ref\":\"#/definitions/voxel_size_cubic\"}"
    "]}}},{\"additionalProperties\":false,\"required\":[\"reference_point\",\"size"
    "_llcs\",\"size\",\"voxel_origin\",\"voxel_type\"],\"properties\":{\"voxel_type\":"
    "{\"$ref\":\"#/definitions/voxel_type\"},\"size\":{\"description\":\"Size of "
    "voxels in file coordinate system.\",\"$ref\":\"#/definitions/voxel_size\"},"
    "\"voxel_origin\":{\"oneOf\":[{\"$ref\":\"#/definitions/voxel_origin_enum\"},{\""
    "description\":\"The base point of the voxel grid. Used together with "
    "<tt>voxel_size</tt> and <tt>voxel_index</tt> to compute actual point c"
    "oordinates.\",\"$ref\":\"#/definitions/voxel_origin_point\"}]},\"size_llcs\":"
    "{\"description\":\"Size of voxels in a locally levelled cartesian "
    "coordinate system (xyz). This is only used for voxels based on a map p"
    "rojection.\",\"$ref\":\"#/definitions/voxel_size\"},\"shape_thresholds\":{\"$r"
    "ef\":\"#/definitions/shape_thresholds\"},\"reference_point\":{\"minItems\":2,"
    "\"type\":\"array\",\"items\":{\"type\":\"number\",\"maximum\":180,\"minimum\":-180},"
    "\"description\":\"Point in WGS84 geodetic decimal degree (EPSG:4326) that"
    " was used to compute the projection distortion parameters. The "
    "coefficient order is latitude, longitude. Only voxels with "
    "corresponding geo_tag, voxel_size and reference_point can be reliably "
    "processed together. This entry is available for voxel files in "
    "projected CRS only.\",\"maxItems\":2}}}],\"title\":\"Voxel "
    "Information\",\"description\":\"Details about the voxels contained in the "
    "file\",\"definitions\":{\"voxel_size_cubic\":{\"type\":\"number\",\"$ref\":\"#/def"
    "initions/edge_length\"},\"voxel_origin_point\":{\"minItems\":3,\"type\":\"arra"
    "y\",\"description\":\"Origin point for all voxel indices in voxel CRS.\",\"i"
    "tems\":{\"type\":\"number\"},\"maxItems\":3},\"voxel_origin_enum\":{\"default\":\""
    "corner\",\"enum\":[\"center\",\"corner\"],\"description\":\"Defines whether the "
    "voxel's center or a corner is placed on CRS origin <tt>(0/0/0)</tt>.\"}"
    ",\"voxel_type\":{\"default\":\"centroid\",\"enum\":[\"center\",\"centroid\",\"index"
    "\"],\"description\":\"Whether a point in a voxel represents its center or "
    "its centroid. If type is <tt>index</tt> there is no point but only an "
    "integer voxel "
    "index.\"},\"edge_length\":{\"type\":\"number\",\"description\":\"Length of voxel"
    " edge [m].\",\"exclusiveMinimum\":true,\"minimum\":0},\"shape_thresholds\":{\""
    "properties\":{\"line\":{\"type\":\"number\",\"description\":\"If the biggest "
    "eigenvalue is bigger than the median eigenvalue * line_threshold, the "
    "voxel is considered a line.\",\"exclusiveMinimum\":true,\"minimum\":1},\"pla"
    "ne\":{\"type\":\"number\",\"exclusiveMaximum\":true,\"minimum\":0,\"maximum\":1,\""
    "description\":\"If the smallest eigenvalue is smaller than the median "
    "eigenvalue * plane_threshold, the voxel is considered a plane.\",\"exclu"
    "siveMinimum\":true}},\"type\":\"object\",\"description\":\"Thresholds used to "
    "compute the voxel's shape_id value.\",\"required\":[\"plane\",\"line\"]},\"vox"
    "el_size\":{\"minItems\":3,\"type\":\"array\",\"items\":{\"$ref\":\"#/definitions/e"
    "dge_length\"},\"description\":\"Size of voxels.\",\"maxItems\":3}}}"
    ,
    /* example: */
    "{\"voxel_type\":\"centroid\",\"size\":[0.5971642834779395,0.5971642834779395"
    ",0.5143705304787237],\"voxel_origin\":\"corner\",\"size_llcs\":[0.5156575252"
    "891171,0.5130835356683303,0.5143705304787237],\"shape_thresholds\":{\"lin"
    "e\":6,\"plane\":0.16},\"reference_point\":[48,16]}"
};

// Settings for waveform averaging
const MetaDataItemInfo RDB_RIEGL_WAVEFORM_AVERAGING_SETTINGS = {
    /* name        */ "riegl.waveform_averaging_settings",
    /* title       */ "Waveform Averaging Settings",
    /* description */ "Settings for waveform averaging",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"required\":[\"num_shots\",\"mta_zone\"],\"type\":\"object\",\"$schema\":\"http:/"
    "/json-schema.org/draft-04/schema#\",\"title\":\"Waveform Averaging "
    "Settings\",\"description\":\"Settings for waveform averaging\",\"properties\""
    ":{\"num_shots\":{\"type\":\"integer\",\"description\":\"Number of consecutive "
    "shots to be used for averaging.\",\"minimum\":1},\"trim\":{\"default\":0,\"typ"
    "e\":\"number\",\"maximum\":0.5,\"description\":\"Percentage for robust averagi"
    "ng.\",\"minimum\":0},\"mta_zone\":{\"type\":\"integer\",\"description\":\"Fixed "
    "MTA zone for averaging.\",\"minimum\":1}}}"
    ,
    /* example: */
    "{\"num_shots\":7,\"trim\":0.05,\"mta_zone\":1}"
};

// Details about waveform files
const MetaDataItemInfo RDB_RIEGL_WAVEFORM_INFO = {
    /* name        */ "riegl.waveform_info",
    /* title       */ "Waveform Information",
    /* description */ "Details about waveform files",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"type\":\"object\",\"$schema\":\"http://json-schema.org/draft-04/schema#\",\""
    "title\":\"Waveform Information\",\"properties\":{\"range_offset_m\":{\"type\":\""
    "number\",\"description\":\"Calibrated device specific range offset for "
    "waveform analysis by system response fitting in meters.\"},\"sample_bloc"
    "k_file\":{\"type\":\"object\",\"properties\":{\"file_uuid\":{\"type\":\"string\",\"d"
    "escription\":\"File's Universally Unique Identifier (RFC "
    "4122)\"},\"file_extension\":{\"type\":\"string\",\"description\":\"Sample block "
    "file extension, without the leading dot\"}},\"required\":[\"file_extension"
    "\"]},\"range_offset_waveform_samples_m\":{\"type\":\"number\",\"description\":\""
    "Calibrated device specific range offset for waveform samples in meters"
    ".\"},\"sample_data_files\":{\"type\":\"array\",\"items\":{\"type\":\"object\",\"prop"
    "erties\":{\"file_uuid\":{\"type\":\"string\",\"description\":\"File's "
    "Universally Unique Identifier (RFC "
    "4122)\"},\"laser_wavelength\":{\"type\":\"number\",\"description\":\"Laser "
    "wavelength in meters (0 = unknown)\",\"exclusiveMinimum\":false,\"minimum\""
    ":0},\"sample_bits\":{\"type\":\"integer\",\"exclusiveMaximum\":false,\"minimum\""
    ":0,\"maximum\":32,\"description\":\"Bitwidth of samples (e.g. 10 bit, 12 bi"
    "t)\",\"exclusiveMinimum\":false},\"file_extension\":{\"type\":\"string\",\"descr"
    "iption\":\"Sample data file extension, without the leading "
    "dot\"},\"sample_interval\":{\"type\":\"number\",\"description\":\"Sampling "
    "interval in seconds\",\"exclusiveMinimum\":false,\"minimum\":0},\"channel\":{"
    "\"type\":\"integer\",\"exclusiveMaximum\":false,\"minimum\":0,\"maximum\":255,\"d"
    "escription\":\"Sample block channel number (255 = invalid)\",\"exclusiveMi"
    "nimum\":false},\"delta_st\":{\"type\":\"number\",\"description\":\"reserved\"},\"c"
    "hannel_name\":{\"type\":\"string\",\"description\":\"Sample block channel name"
    "\"}},\"required\":[\"channel\",\"channel_name\",\"sample_interval\",\"sample_bit"
    "s\",\"laser_wavelength\",\"delta_st\",\"file_extension\"]}}},\"description\":\"D"
    "etails about waveform "
    "files\",\"required\":[\"sample_block_file\",\"sample_data_files\"]}"
    ,
    /* example: */
    "{\"range_offset_m\":3.1415,\"sample_block_file\":{\"file_uuid\":\"93a03615-66"
    "c0-4bea-8ff8-c577378fe660\",\"file_extension\":\"sbx\"},\"sample_data_files\""
    ":[{\"sample_bits\":12,\"laser_wavelength\":0,\"file_extension\":\"sp0\",\"sampl"
    "e_interval\":1.00503e-09,\"channel\":0,\"delta_st\":0,\"file_uuid\":\"da084413"
    "-e3e8-4655-a122-071de8490d8e\",\"channel_name\":\"high_power\"},{\"sample_bi"
    "ts\":12,\"laser_wavelength\":0,\"file_extension\":\"sp1\",\"sample_interval\":1"
    ".00503e-09,\"channel\":1,\"delta_st\":0,\"file_uuid\":\"93585b5e-5ea9-43a1-94"
    "7b-e7ba3be642d2\",\"channel_name\":\"low_power\"},{\"sample_bits\":12,\"laser_"
    "wavelength\":0,\"file_extension\":\"sp5\",\"sample_interval\":1.00503e-09,\"ch"
    "annel\":5,\"delta_st\":0,\"file_uuid\":\"9d2298c4-1036-464f-b5cb-1cf8e517f3a"
    "0\",\"channel_name\":\"wwf\"}],\"range_offset_waveform_samples_m \":7.283}"
};

// Scanner settings for waveform output
const MetaDataItemInfo RDB_RIEGL_WAVEFORM_SETTINGS = {
    /* name        */ "riegl.waveform_settings",
    /* title       */ "Waveform Settings",
    /* description */ "Scanner settings for waveform output",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"type\":\"array\",\"items\":{\"type\":\"object\",\"required\":[\"sbl_name\",\"enabl"
    "ed\",\"channel_idx_mask\"],\"properties\":{\"pass_rng_greater\":{\"type\":\"numb"
    "er\",\"description\":\"Threshold for range greater than "
    "[m]\"},\"pass_ampl_greater\":{\"type\":\"number\",\"description\":\"Threshold "
    "for amplitude greater than "
    "[dB]\"},\"channel_idx_mask\":{\"type\":\"integer\",\"description\":\"Bit mask "
    "for channels which belong to sbl_name: Channel 0 = Bit0, Channel 1 = "
    "Bit1, ...\"},\"pass_rng_less\":{\"type\":\"number\",\"description\":\"Threshold "
    "for range less than "
    "[m]\"},\"sbl_name\":{\"type\":\"string\",\"description\":\"Name of sample block,"
    " e.g.: wfm, "
    "wwf\"},\"smart_enabled\":{\"type\":\"boolean\",\"description\":\"Smart waveform "
    "output enabled\"},\"pass_dev_greater\":{\"type\":\"integer\",\"description\":\"T"
    "hreshold for deviation greater than "
    "[1]\"},\"pass_dev_less\":{\"type\":\"integer\",\"description\":\"Threshold for "
    "deviation less than "
    "[1]\"},\"pass_ampl_less\":{\"type\":\"number\",\"description\":\"Threshold for "
    "amplitude less than "
    "[dB]\"},\"logic_expression\":{\"type\":\"string\",\"description\":\"Logic "
    "expression of smart waveforms "
    "filter\"},\"enabled\":{\"type\":\"boolean\",\"description\":\"Waveform output en"
    "abled\"}}},\"$schema\":\"http://json-schema.org/draft-04/schema#\",\"descrip"
    "tion\":\"Scanner settings for waveform output\",\"title\":\"Waveform "
    "Settings\"}"
    ,
    /* example: */
    "[{\"pass_rng_greater\":9.27,\"pass_ampl_greater\":1.0,\"pass_rng_less\":13.1"
    "1,\"sbl_name\":\"wfm\",\"smart_enabled\":true,\"channel_idx_mask\":11,\"pass_am"
    "pl_less\":5.0,\"enabled\":true},{\"enabled\":false,\"channel_idx_mask\":32,\"s"
    "bl_name\":\"wwf\"}]"
};

// Window analysis data estimated from scandata and resulting filter parameters
const MetaDataItemInfo RDB_RIEGL_WINDOW_ANALYSIS = {
    /* name        */ "riegl.window_analysis",
    /* title       */ "Window Analysis",
    /* description */ "Window analysis data estimated from scandata and resulting filter parameters",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"required\":[\"result\",\"filter\",\"settings\"],\"type\":\"object\",\"$schema\":\""
    "http://json-schema.org/draft-04/schema#\",\"title\":\"Window "
    "Analysis\",\"description\":\"Window analysis data estimated from scandata "
    "and resulting filter parameters\",\"properties\":{\"settings\":{\"type\":\"obj"
    "ect\",\"required\":[\"range\",\"amplitude\"],\"properties\":{\"range\":{\"type\":\"o"
    "bject\",\"required\":[\"sigma_factor\",\"additive_value\"],\"properties\":{\"sig"
    "ma_factor\":{\"type\":\"number\"},\"additive_value\":{\"type\":\"number\"}}},\"amp"
    "litude\":{\"type\":\"object\",\"required\":[\"sigma_factor\",\"additive_value\"],"
    "\"properties\":{\"sigma_factor\":{\"type\":\"number\"},\"additive_value\":{\"type"
    "\":\"number\"}}}}},\"filter\":{\"type\":\"object\",\"required\":[\"angle\",\"range_m"
    "in\",\"range_max\",\"amplitude_max\"],\"properties\":{\"angle\":{\"type\":\"array\""
    ",\"items\":{\"type\":\"number\"},\"description\":\"[deg]\"},\"range_max\":{\"type\":"
    "\"array\",\"items\":{\"type\":\"number\"},\"description\":\"[m]\"},\"range_min\":{\"t"
    "ype\":\"array\",\"items\":{\"type\":\"number\"},\"description\":\"[m]\"},\"amplitude"
    "_max\":{\"type\":\"array\",\"items\":{\"type\":\"number\"},\"description\":\"[dB]\"}}"
    "},\"result\":{\"type\":\"object\",\"required\":[\"angle\",\"range_mean\",\"range_si"
    "gma\",\"amplitude_mean\",\"amplitude_sigma\",\"amplitude_offset\"],\"propertie"
    "s\":{\"angle\":{\"type\":\"array\",\"items\":{\"type\":\"number\"},\"description\":\"["
    "deg]\"},\"amplitude_offset\":{\"type\":\"array\",\"items\":{\"type\":\"number\"},\"d"
    "escription\":\"[dB]\"},\"amplitude_mean\":{\"type\":\"array\",\"items\":{\"type\":\""
    "number\"},\"description\":\"[dB]\"},\"amplitude_sigma\":{\"type\":\"array\",\"item"
    "s\":{\"type\":\"number\"},\"description\":\"[dB]\"},\"timestamp\":{\"type\":\"array\""
    ",\"items\":{\"type\":\"number\"},\"description\":\"[s]\"},\"range_mean\":{\"type\":\""
    "array\",\"items\":{\"type\":\"number\"},\"description\":\"[m]\"},\"range_sigma\":{\""
    "type\":\"array\",\"items\":{\"type\":\"number\"},\"description\":\"[m]\"}}}}}"
    ,
    /* example: */
    "{\"settings\":{\"range\":{\"sigma_factor\":8,\"additive_value\":0.1},\"amplitud"
    "e\":{\"sigma_factor\":4,\"additive_value\":1.0}},\"filter\":{\"angle\":[14.9,15"
    ".0,15.1,15.2,15.3,15.4,15.5,15.6,15.7,15.8,15.9],\"range_max\":[0.424,0."
    "425,0.426,0.427,0.428,0.428,0.429,0.43,0.431,0.431,0.432],\"range_min\":"
    "[-0.208,-0.21,-0.212,-0.214,-0.216,-0.218,-0.219,-0.221,-0.223,-0.225,"
    "-0.227],\"amplitude_max\":[8.04,8.01,7.99,7.96,7.93,7.9,7.88,7.85,7.83,7"
    ".8,7.78]},\"result\":{\"angle\":[14.9,15.0,15.1,15.2,15.3,15.4,15.5,15.6,1"
    "5.7,15.8,15.9],\"amplitude_offset\":[1.9,1.9],\"amplitude_mean\":[5.347396"
    ",5.263155,5.224655,5.179926,5.097782,5.116479,5.051756,4.983473,5.0078"
    "85,5.002441,4.982],\"amplitude_sigma\":[0.4272844,0.4298479,0.4236816,0."
    "4283583,0.4362353,0.4315141,0.4373984,0.4472798,0.4346001,0.4345487,0."
    "4540681],\"timestamp\":[408.4441,411.4443],\"range_mean\":[0.1105621,0.107"
    "9564,0.1087088,0.1067261,0.1054582,0.1090412,0.102871,0.1019044,0.1051"
    "523,0.1058445,0.1031261],\"range_sigma\":[0.01869652,0.02151435,0.017479"
    "69,0.01918765,0.01945776,0.01934862,0.01955329,0.02225589,0.02229977,0"
    ".01899122,0.02009433]}}"
};

// Correction parameters for window glass echoes
const MetaDataItemInfo RDB_RIEGL_WINDOW_ECHO_CORRECTION = {
    /* name        */ "riegl.window_echo_correction",
    /* title       */ "Window Echo Correction",
    /* description */ "Correction parameters for window glass echoes",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"required\":[\"amplitude\",\"range\",\"slices\"],\"type\":\"object\",\"$schema\":\""
    "http://json-schema.org/draft-04/schema#\",\"title\":\"Window Echo "
    "Correction\",\"description\":\"Correction parameters for window glass echo"
    "es\",\"properties\":{\"range\":{\"required\":[\"minimum\",\"maximum\",\"entries\"],"
    "\"type\":\"object\",\"description\":\"Range axis of correction table\",\"proper"
    "ties\":{\"maximum\":{\"type\":\"number\",\"maximum\":2.0,\"description\":\"Maximum"
    " range in "
    "m\",\"minimum\":-2.0},\"entries\":{\"type\":\"integer\",\"description\":\"Number "
    "of range entries\",\"minimum\":1},\"minimum\":{\"type\":\"number\",\"maximum\":2."
    "0,\"description\":\"Minimum range in m\",\"minimum\":-2.0}}},\"slices\":{\"type"
    "\":\"array\",\"items\":{\"required\":[\"amplitude\",\"table\"],\"type\":\"object\",\"d"
    "escription\":\"Window echo correction parameter slice\",\"properties\":{\"am"
    "plitude\":{\"type\":\"number\",\"description\":\"Window echo amplitude of "
    "slice in "
    "dB\"},\"table\":{\"minItems\":1,\"type\":\"array\",\"description\":\"Correction "
    "table (dimension defined by the 'amplitude' and 'range' "
    "objects)\",\"items\":{\"minItems\":1,\"type\":\"array\",\"description\":\"Table "
    "row (= amplitude "
    "axis)\",\"items\":{\"minItems\":3,\"type\":\"array\",\"description\":\"Table "
    "column (= range axis)\",\"items\":{\"type\":\"number\",\"description\":\"Table "
    "cell (item 0: amplitude in dB, 1: range in m, 2: flags)\"},\"maxItems\":3"
    "}}}}}},\"amplitude\":{\"required\":[\"minimum\",\"maximum\",\"entries\"],\"type\":"
    "\"object\",\"description\":\"Amplitude axis of correction "
    "table\",\"properties\":{\"maximum\":{\"type\":\"number\",\"description\":\"Maximum"
    " amplitude in "
    "dB\",\"minimum\":0.0},\"entries\":{\"type\":\"integer\",\"description\":\"Number "
    "of amplitude entries\",\"minimum\":1},\"minimum\":{\"type\":\"number\",\"descrip"
    "tion\":\"Minimum amplitude in dB\",\"minimum\":0.0}}}}}"
    ,
    /* example: */
    "{\"range\":{\"maximum\":1.5060822940732335,\"entries\":128,\"minimum\":-1.5060"
    "822940732335},\"slices\":[{\"amplitude\":1.5,\"table\":[[[6.23,0.3535,1]],[["
    "5.54,0.72375,1]]]},{\"amplitude\":2.0,\"table\":[[[6.23,0.3535,1]],[[5.54,"
    "0.72375,1]]]}],\"amplitude\":{\"maximum\":20,\"entries\":128,\"minimum\":2}}"
};

// Table of all meta data item details records
const MetaDataItemInfo* const RDB_META_DATA_ITEMS[] = {
    &RDB_RIEGL_ANGULAR_NOTCH_FILTER,
    &RDB_RIEGL_ATMOSPHERE,
    &RDB_RIEGL_BEAM_GEOMETRY,
    &RDB_RIEGL_CONTROL_OBJECT_CATALOG,
    &RDB_RIEGL_CONTROL_OBJECT_REFERENCE_FILE,
    &RDB_RIEGL_DETECTION_PROBABILITY,
    &RDB_RIEGL_DEVICE,
    &RDB_RIEGL_DEVICE_GEOMETRY,
    &RDB_RIEGL_DEVICE_GEOMETRY_PASSIVE_CHANNEL,
    &RDB_RIEGL_DEVICE_OUTPUT_LIMITS,
    &RDB_RIEGL_DEVICES,
    &RDB_RIEGL_ECHO_INFO,
    &RDB_RIEGL_EXPONENTIAL_DECOMPOSITION,
    &RDB_RIEGL_GAUSSIAN_DECOMPOSITION,
    &RDB_RIEGL_GEO_TAG,
    &RDB_RIEGL_GEOMETRIC_SCALE_FACTOR,
    &RDB_RIEGL_GEOREFERENCING_PARAMETERS,
    &RDB_RIEGL_ITEM_NAMES,
    &RDB_RIEGL_LICENSES,
    &RDB_RIEGL_MTA_SETTINGS,
    &RDB_RIEGL_NEAR_RANGE_CORRECTION,
    &RDB_RIEGL_NOISE_ESTIMATES,
    &RDB_RIEGL_NOTCH_FILTER,
    &RDB_RIEGL_PIXEL_INFO,
    &RDB_RIEGL_PLANE_PATCH_MATCHING,
    &RDB_RIEGL_PLANE_PATCH_STATISTICS,
    &RDB_RIEGL_PLANE_SLOPE_CLASS_INFO,
    &RDB_RIEGL_POINT_ATTRIBUTE_GROUPS,
    &RDB_RIEGL_POINTCLOUD_INFO,
    &RDB_RIEGL_POSE_ESTIMATION,
    &RDB_RIEGL_POSE_SENSORS,
    &RDB_RIEGL_PULSE_POSITION_MODULATION,
    &RDB_RIEGL_RANGE_STATISTICS,
    &RDB_RIEGL_RECEIVER_INTERNALS,
    &RDB_RIEGL_REFLECTANCE_CALCULATION,
    &RDB_RIEGL_REFLECTANCE_CORRECTION,
    &RDB_RIEGL_SCAN_PATTERN,
    &RDB_RIEGL_SHOT_INFO,
    &RDB_RIEGL_STORED_FILTERS,
    &RDB_RIEGL_SYSTEM_DESCRIPTION,
    &RDB_RIEGL_TEMPERATURE_CALCULATION,
    &RDB_RIEGL_TIME_BASE,
    &RDB_RIEGL_TRAJECTORY_INFO,
    &RDB_RIEGL_TTIP_CONFIGURATION,
    &RDB_RIEGL_VERTEX_INFO,
    &RDB_RIEGL_VOXEL_INFO,
    &RDB_RIEGL_WAVEFORM_AVERAGING_SETTINGS,
    &RDB_RIEGL_WAVEFORM_INFO,
    &RDB_RIEGL_WAVEFORM_SETTINGS,
    &RDB_RIEGL_WINDOW_ANALYSIS,
    &RDB_RIEGL_WINDOW_ECHO_CORRECTION
};
const size_t RDB_META_DATA_ITEM_COUNT(
    sizeof(RDB_META_DATA_ITEMS) / sizeof(RDB_META_DATA_ITEMS[0])
);

#endif // RDB_81EBCB65_FE94_471E_8D25_2B4D71CC91B1
